@@ -42,31 +42,49 @@
 :- discontiguous
 	prolog_message/3.
 
+%%	translate_message(+Term)// is det.
+%
+%	Translate a message Term into message lines. The produced lines
+%	is a list of
+%	
+%	    * nl
+%	    Emit a newline
+%	    * Fmt-Args
+%	    Emit the result of format(Fmt, Args)
+%	    * Fmt
+%	    Emit the result of format(Fmt)
+%	    * flush
+%	    Used only as last element of the list.   Simply flush the
+%	    output instead of producing a final newline.
+
 translate_message(Term) -->
+	translate_message2(Term), !.
+
+translate_message2(Term) -->
 	{var(Term)}, !,
 	[ 'Unknown message: ~p'-[Term] ].
-translate_message(Term) -->
+translate_message2(Term) -->
 	prolog:message(Term).
-translate_message(Term) -->
+translate_message2(Term) -->
 	prolog_message(Term).
-translate_message(error(resource_error(stack), Name)) -->
+translate_message2(error(resource_error(stack), Name)) -->
 	[ 'Out of ~w stack'-[Name] ].
-translate_message(error(resource_error(Missing), _)) -->
+translate_message2(error(resource_error(Missing), _)) -->
 	[ 'Not enough resources: ~w'-[Missing] ].
-translate_message(error(ISO, SWI)) -->
+translate_message2(error(ISO, SWI)) -->
 	swi_context(SWI),
 	term_message(ISO),
 	swi_extra(SWI).
-translate_message('$aborted') -->
+translate_message2('$aborted') -->
 	[ 'Execution Aborted' ].
-translate_message(message_lines(Lines), L, T) :- % deal with old C-warning()
+translate_message2(message_lines(Lines), L, T) :- % deal with old C-warning()
 	make_message_lines(Lines, L, T).
-translate_message(format(Fmt, Args)) -->
+translate_message2(format(Fmt, Args)) -->
 	[ Fmt-Args ].
-translate_message(Term) -->
+translate_message2(Term) -->
 	{ Term = error(_, _) },
 	[ 'Unknown exception: ~p'-[Term] ].
-translate_message(Term) -->
+translate_message2(Term) -->
 	[ 'Unknown message: ~p'-[Term] ].
 
 make_message_lines([], T, T) :- !.
@@ -319,6 +337,12 @@ prolog_message(io_warning(Stream, Message)) -->
 	[ 'stream ~p: ~w'-[Stream, Message] ].
 prolog_message(option_usage(pldoc)) -->
 	[ 'Usage: --pldoc[=port]' ].
+prolog_message(interrupt(begin)) -->
+	[ 'Action (h for help) ? ', flush ].
+prolog_message(interrupt(end)) -->
+	[ 'continue' ].
+prolog_message(interrupt(trace)) -->
+	[ 'continue (trace mode)' ].
 
 
 		 /*******************************
@@ -484,6 +508,10 @@ prolog_message(version) -->
 	  Patch is Version mod 100
 	},
 	[ '~w.~w.~w'-[Major, Minor, Patch] ].
+prolog_message(address_bits) -->
+	{ current_prolog_flag(address_bits, Bits)
+	}, !,
+	[ '~d bits, '-[Bits] ].
 prolog_message(threads) -->
 	{ current_prolog_flag(threads, true)
 	}, !,
@@ -491,7 +519,7 @@ prolog_message(threads) -->
 prolog_message(threads) -->
 	[].
 prolog_message(copyright) -->
-	[ 'Copyright (c) 1990-2007 University of Amsterdam.', nl,
+	[ 'Copyright (c) 1990-2008 University of Amsterdam.', nl,
 	  'SWI-Prolog comes with ABSOLUTELY NO WARRANTY. This is free software,', nl,
 	  'and you are welcome to redistribute it under certain conditions.', nl,
 	  'Please visit http://www.swi-prolog.org for details.'
@@ -501,6 +529,7 @@ prolog_message(author) -->
 prolog_message(welcome) -->
 	[ 'Welcome to SWI-Prolog (' ],
 	prolog_message(threads),
+	prolog_message(address_bits),
 	['Version ' ],
 	prolog_message(version),
 	[ ')', nl ],
@@ -529,21 +558,60 @@ prolog_message(var_query(_)) -->
 prolog_message(close_on_abort(Stream)) -->
 	[ 'Abort: closed stream ~p'-[Stream] ].
 
-prolog_message(query(no)) -->
-	[ nl, 'No' ].
-prolog_message(query(yes)) -->
-	[ nl, 'Yes' ].
-prolog_message(query(Yes, Bindings)) -->
-	bindings(Bindings),
-	prompt(Yes, Bindings).
-prolog_message(query(eof)) -->
-	[ nl ].
+prolog_message(query(QueryResult)) -->
+	query_result(QueryResult).
 
-prompt(yes, []) --> !,
-	[ nl, 'Yes' ].
-prompt(yes, _) --> !.
-prompt(more, []) --> !,
-	[ 'More? ', flush ].
+query_result(no) -->		% failure
+	[ 'fail.' ],
+	extra_line.
+query_result(yes([])) --> !,	% prompt_alternatives_on: groundness
+	[ 'true.' ],
+	extra_line.
+query_result(yes(Residuals)) -->
+	residuals(Residuals),
+	extra_line.
+query_result(done) -->		% user typed <CR>
+	extra_line.
+query_result(yes(Bindings, Residuals)) -->
+	bindings(Bindings),
+	bind_res_sep(Bindings, Residuals),
+	residuals(Residuals),
+	prompt(yes, Bindings, Residuals).
+query_result(more(Bindings, Residuals)) -->
+	bindings(Bindings),
+	bind_res_sep(Bindings, Residuals),
+	residuals(Residuals),
+	prompt(more, Bindings, Residuals).
+query_result(help) -->
+	[ nl, 'Actions:', nl, nl,
+	  '; (n, r, space, TAB): redo    t:          trace & redo', nl,
+	  'b:                    break   c (a, RET): exit', nl,
+	  'w:                    write   p           print', nl,
+	  'h (?):                help',
+	  nl, nl
+	].
+query_result(action) -->
+	[ 'Action? ', flush ].
+query_result(confirm) -->
+	[ 'Please answer \'y\' or \'n\'? ', flush ].
+query_result(eof) -->
+	[ nl ].
+query_result(toplevel_open_line) -->
+	[].
+
+prompt(Answer, [], []) --> !,
+	prompt(Answer, empty).
+prompt(Answer, _, _) --> !,
+	prompt(Answer, non_empty).
+
+prompt(yes, empty) --> !,
+	[ 'true.' ],
+	extra_line.
+prompt(yes, _) --> !,
+	[ '.' ],
+	extra_line.
+prompt(more, empty) --> !,
+	[ 'true ', flush ].
 prompt(more, _) --> !,
 	[ ' ', flush ].
 
@@ -556,18 +624,34 @@ bindings([Name = Value|T]) -->
 	  ;   Cont = (,)
 	  )
 	},
-	[ nl, '~w = ~W~w'-[Name, Value, Options, Cont] ],
+	[ '~w = ~W~w'-[Name, Value, Options, Cont] ],
+	(   {T == []}
+	->  []
+	;   [nl]
+	),
 	bindings(T).
 
-prolog_message(query(help)) -->
-	[ nl, 'Actions:', nl, nl,
-	  '; (n, r):     redo    t:                 trace & redo', nl,
-	  'b:            break   c (a, RET, space): continue', nl,
-	  'w:            write   p                  print', nl,
-	  'h (?):        help', nl, nl
-	].
-prolog_message(query(action)) -->
-	[ 'Action? ', flush ].
+residuals([]) --> !, [].
+residuals([G|Gs]) -->
+	(   { Gs \== [] }
+	->  [ '~q,'-[G], nl ],
+	    residuals(Gs)
+	;   [ '~q'-[G] ]
+	).
+	
+bind_res_sep(_, []) --> !,
+	[].
+bind_res_sep([], _) --> !,
+	[].
+bind_res_sep(_, _) -->
+	[','-[], nl].
+
+extra_line -->
+	{ current_prolog_flag(toplevel_extra_white_line, true) }, !,
+	['~N'-[]].
+extra_line -->
+	[].
+
 prolog_message(if_tty(Text)) -->
 	(   {current_prolog_flag(tty_control, true)}
 	->  [ at_same_line, '~w'-[Text] ]
@@ -577,8 +661,6 @@ prolog_message(halt(Reason)) -->
 	[ '~w: halt'-[Reason] ].
 prolog_message(no_action(Char)) -->
 	[ 'Unknown action: ~c (h for help)'-[Char], nl ].
-prolog_message(query(confirm)) -->
-	[ 'Please answer \'y\' or \'n\'? ', flush ].
 
 prolog_message(history(help(Show, Help))) -->
 	[ 'History Commands:', nl, 
@@ -820,8 +902,8 @@ print_system_message(_, Level, Lines) :-
 	prefix(Level, LinePrefix, Stream), !,
 	print_message_lines(Stream, LinePrefix, Lines).
 	
-prefix(error,	      'ERROR: (~w:~d):~n',   '\t', '', 0.5, user_error).
-prefix(warning,	      'Warning: (~w:~d):~n', '\t', '', 0,   user_error).
+prefix(error,	      'ERROR: ~w:~d:~n',   '\t', '', 0.5, user_error).
+prefix(warning,	      'Warning: ~w:~d:~n', '\t', '', 0,   user_error).
 
 prefix(help,	      '',          user_error).
 prefix(query,	      '',          user_error).
