@@ -469,6 +469,8 @@ generate_current_predicate(Name, Module, Head) :-
 	predicate_property/2,
 	'$predicate_property'/2.
 
+:- '$iso'(predicate_property/2).
+
 predicate_property(Pred, Property) :-
 	Property == undefined, !,
 	(   Pred = Module:Head,
@@ -526,6 +528,8 @@ predicate_property(Pred, Property) :-
 	'$get_predicate_attribute'(Pred, number_of_clauses, N).
 '$predicate_property'(noprofile, Pred) :-
 	'$get_predicate_attribute'(Pred, noprofile, 1).
+'$predicate_property'(iso, Pred) :-
+	'$get_predicate_attribute'(Pred, iso, 1).
 
 :- index(clause_property(0, 1)).
 
@@ -703,7 +707,11 @@ on_signal(Signal, Old, New) :-
 	integer(Signal), !,
 	'$on_signal'(Signal, _Name, Old, New).
 on_signal(Signal, _Old, _New) :-
-	throw(error(type_error(signal, Signal), on_signal/3)).
+	(   var(Signal)
+	->  Err = instantiation_error
+	;   Err = type_error(signal, Signal)
+	),
+	throw(error(Err, context(on_signal/3, _))).
 
 current_signal(Name, Id, Handler) :-
 	between(1, 32, Id),
@@ -806,6 +814,8 @@ default_module(Me, Super) :-
 %
 %	Is true when N is the length of List.
 
+:- '$iso'((length/2)).
+
 length(List, Length) :-
 	(   nonvar(Length)
 	->  '$length'(List, Length)
@@ -829,6 +839,8 @@ length3([_|List], N, N0) :-
 		 *	       TERM		*
 		 *******************************/
 
+:- '$iso'((numbervars/3)).
+
 %	numbervars(+Term, +StartIndex, -EndIndex)
 %	
 %	Number all unbound variables in Term   using  '$VAR'(N), where the
