@@ -249,9 +249,9 @@ set_associated_file :-
 	file_directory_name(File, Dir),
 	working_directory(_, Dir),
 	set_prolog_flag(associated_file, File),
-	atom_concat('SWI-Prolog -- ', File, Title),
-	(   '$c_current_predicate'(_, system:window_title(_, _))
-	->  system:window_title(_, Title)
+	(   current_predicate(system:window_title/2)
+	->  atom_concat('SWI-Prolog -- ', File, Title),
+	    system:window_title(_, Title)
 	;   true
 	).
 set_associated_file.
@@ -283,8 +283,8 @@ start_pldoc.
 
 %%	load_associated_file
 %	
-%	Load the file-name set by set_associated_file/0 from the
-%	commandline arguments. Not the expand(false) to avoid expanding
+%	Load  the  file-name  set  by   set_associated_file/0  from  the
+%	commandline arguments. Note the expand(false) to avoid expanding
 %	special characters in the filename.
 
 load_associated_file :-
@@ -330,6 +330,7 @@ initialise_prolog :-
 	once(print_predicate(_, [print], PrintOptions)),
 	set_prolog_flag(toplevel_print_options, PrintOptions),
 	set_prolog_flag(prompt_alternatives_on, determinism),
+	set_prolog_flag(toplevel_extra_white_line, true),
 	'$set_debugger_print_options'(print),
 	'$run_at_initialization',
 	'$load_system_init_file',
@@ -585,29 +586,69 @@ subst_chars([H|T]) -->
 %	for alternatives. =groundness= gives   the  classical behaviour,
 %	=determinism= is considered more adequate and informative.
 
-write_bindings(Bindings0, Det) :-
-	bind_vars(Bindings0),
-	filter_bindings(Bindings0, Bindings),
-	write_bindings2(Bindings, Det).
+write_bindings(Bindings, Det) :-
+	'$attributed'(Bindings), !,
+	copy_term(Bindings, Bindings1, Residuals0),
+	'$module'(TypeIn, TypeIn),
+	omit_qualifiers(Residuals0, TypeIn, Residuals),
+	bind_vars(Bindings1),
+	filter_bindings(Bindings1, Bindings2),
+	write_bindings2(Bindings2, Residuals, Det).
+write_bindings(Bindings, Det) :-
+	bind_vars(Bindings),
+	filter_bindings(Bindings, Bindings1),
+	write_bindings2(Bindings1, [], Det).
 
-write_bindings2([], _) :-
+write_bindings2([], Residuals, _) :-
 	current_prolog_flag(prompt_alternatives_on, groundness), !,
-	print_message(query, query(yes)).
-write_bindings2(Bindings, true) :-
+	print_message(query, query(yes(Residuals))).
+write_bindings2(Bindings, Residuals, true) :-
 	current_prolog_flag(prompt_alternatives_on, determinism), !,
-	print_message(query, query(yes, Bindings)).
-write_bindings2(Bindings, _Det) :-
+	print_message(query, query(yes(Bindings, Residuals))).
+write_bindings2(Bindings, Residuals, _Det) :-
 	repeat,
-	    print_message(query, query(more, Bindings)),
+	    print_message(query, query(more(Bindings, Residuals))),
 	    get_respons(Action),
 	(   Action == redo
 	->  !, fail
 	;   Action == show_again
 	->  fail
 	;   !,
-	    print_message(query, query(yes))
+	    print_message(query, query(done))
 	).
 
+
+%%	omit_qualifiers(+QGoals, +TypeIn, -Goals) is det.
+%
+%	Omit unneeded module qualifiers  from   QGoals  relative  to the
+%	given module TypeIn.
+
+omit_qualifiers([], _, []).
+omit_qualifiers([Goal0|Goals0], TypeIn, [Goal|Goals]) :-
+	omit_qualifier(Goal0, TypeIn, Goal),
+	omit_qualifiers(Goals0, TypeIn, Goals).
+
+omit_qualifier(M:G0, TypeIn, G) :-
+	predicate_property(TypeIn:G0, imported_from(M)), !,
+	omit_meta_qualifiers(G0, TypeIn, G).
+omit_qualifier(_:G0, TypeIn, G) :-
+	predicate_property(G0, built_in), !,
+	omit_meta_qualifiers(G0, TypeIn, G).
+omit_qualifier(M:G0, _, M:G) :-
+	atom(M), !,
+	omit_meta_qualifiers(G0, M, G).
+omit_qualifier(G0, TypeIn, G) :-
+	omit_meta_qualifiers(G0, TypeIn, G).
+
+omit_meta_qualifiers(freeze(V, QGoal), TypeIn, freeze(V, Goal)) :-
+	callable(QGoal), !,
+	omit_qualifier(QGoal, TypeIn, Goal).
+omit_meta_qualifiers(when(Cond, QGoal), TypeIn, when(Cond, Goal)) :-
+	callable(QGoal), !,
+	omit_qualifier(QGoal, TypeIn, Goal).
+omit_meta_qualifiers(G, _, G).
+
+
 %%	bind_vars(+Bindings)
 %	
 %	Bind variables to '$VAR'(Name), so they are printed by the names
@@ -654,14 +695,15 @@ answer_respons(Char, again) :-
 	memberchk(Char, "?h"), !,
 	print_message(help, query(help)).
 answer_respons(Char, redo) :-
-	memberchk(Char, ";nrNR"), !,
+	memberchk(Char, ";nrNR \t"), !,
 	print_message(query, if_tty(';')).
 answer_respons(Char, redo) :-
 	memberchk(Char, "tT"), !,
 	trace,
 	print_message(query, if_tty('; [trace]')).
 answer_respons(Char, continue) :-
-	memberchk(Char, "ca \n\ryY"), !.
+	memberchk(Char, "ca\n\ryY."), !,
+	print_message(query, if_tty('.')).
 answer_respons(0'b, show_again) :- !,
 	break.
 answer_respons(Char, show_again) :-
@@ -675,12 +717,10 @@ answer_respons(Char, again) :-
 	print_message(query, no_action(Char)).
 
 print_predicate(0'w, [write], [ quoted(true),
-				attributes(write),
 				priority(699)
 			      ]).
 print_predicate(0'p, [print], [ quoted(true),
 				portray(true),
-				attributes(portray),
 				max_depth(10),
 				priority(699)
 			      ]).
