@@ -1,4 +1,4 @@
-/*  $Id: edit.pl,v 1.24 2006/03/24 14:36:53 jan Exp $
+/*  $Id$
 
     Part of SWI-Prolog
 
@@ -35,13 +35,15 @@
 	  ]).
 :- use_module(library(lists), [append/3, member/2, nth1/3]).
 :- use_module(library(make), [make/0]).
+:- set_prolog_flag(generate_debug_info, false).
+
+/** <module> Editor interface
 
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 This module implements the generic editor  interface. It consists of two
 extensible parts with little  in  between.   The  first  part deals with
 translating the input into source-location, and the second with starting
 an editor.
-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+*/
 
 :- multifile
 	locate/3,			% +Partial, -FullSpec, -Location
@@ -51,7 +53,7 @@ an editor.
 	edit_command/2,			% +Editor, -Command
 	load/0.				% provides load-hooks
 
-%	edit(+Spec)
+%%	edit(+Spec)
 %
 %	Edit indicated object.
 
@@ -66,9 +68,14 @@ edit(Spec) :-
 	do_select_location(Pairs, Spec, Location),
 	do_edit_source(Location).
 
-%	edit
+%%	edit
 %	
-%	Load associated or script file
+%	Edit associated or script file.  This is the Prolog file opened
+%	by double-clicking or the file loaded using
+%	
+%	==
+%	% pl -s file.pl
+%	==
 
 edit :-
 	'$option'(script_file, OsFile, OsFile),
@@ -79,14 +86,14 @@ edit :-
 	current_prolog_flag(associated_file, File), !,
 	edit(file(File)).
 edit :-
-	throw(error(existence_error(flag, associated_file), _)).
+	throw(error(context_error(edit, no_default_file), _)).
 
 
 		 /*******************************
 		 *	      LOCATE		*
 		 *******************************/
 
-%	locate(+Spec, -FullSpec, -Location)
+%%	locate(+Spec, -FullSpec, -Location)
 
 locate(Path, file(Path), [file(Path)]) :-
 	atom(Path),
@@ -119,11 +126,15 @@ locate(Name, FullSpec, Location) :-
 	locate(Name/_, FullSpec, Location).
 locate(Name/Arity, Module:Name/Arity, Location) :-
 	locate(Module:Name/Arity, Location).
+locate(Name//DCGArity, FullSpec, Location) :-
+	(   integer(DCGArity)
+	->  Arity is DCGArity+2,
+	    locate(Name/Arity, FullSpec, Location)
+	;   locate(Name/_, FullSpec, Location) % demand arity >= 2
+	).
 locate(Name/Arity, library(File), [file(PlPath)]) :-
 	atom(Name),
-	'$autoload':load_library_index,
-	'$autoload':library_index(Head, _, Path),
-	functor(Head, Name, Arity),
+	'$in_library'(Name, Arity, Path),
 	(   absolute_file_name(library(.),
 			       [ file_type(directory),
 				 solutions(all)
@@ -150,7 +161,7 @@ locate(Spec, module(Spec), Location) :-
 locate(Spec, Spec, Location) :-
 	locate(Spec, Location).
 
-%	locate(+Spec, -Location)
+%%	locate(+Spec, -Location)
 %
 %	Locate object from the specified location.
 
@@ -194,7 +205,7 @@ locate(clause(Ref, _PC), [file(File), line(Line)]) :- % TBD: use clause
 		 *	       EDIT		*
 		 *******************************/
 
-%	do_edit_source(+Location)
+%%	do_edit_source(+Location)
 %	
 %	Actually call the editor to edit Location, a list of Name(Value)
 %	that contains file(File) and may contain line(Line). First the
@@ -214,8 +225,8 @@ do_edit_source(Location) :-		% PceEmacs
 	current_prolog_flag(gui, true), !,
 	memberchk(file(File), Location),
 	(   memberchk(line(Line), Location)
-	->  Goal = emacs(File:Line)
-	;   Goal = emacs(File)
+	->  Goal = in_pce_thread(emacs(File:Line))
+	;   Goal = in_pce_thread(emacs(File))
 	),
 	Goal.
 do_edit_source(Location) :-		% External editor
@@ -262,7 +273,7 @@ external_edit_command(Location, Command) :-
 pceemacs(pce_emacs).
 pceemacs(built_in).
 
-%	editor(-Editor)
+%%	editor(-Editor)
 %	
 %	Determine the external editor to run.
 
@@ -284,16 +295,15 @@ editor(notepad) :-
 editor(_) :-				% No luck
 	throw(error(existence_error(editor), _)).
 
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-edit_command(+Editor, -Command)
+%%	edit_command(+Editor, -Command)
+%
+%	This predicate should specify the shell-command called to invoke
+%	the user's editor. The following substitutions will be made:
+%
+%		| %e | Path name of the editor		  |
+%		| %f | Path name of the file to be edited |
+%		| %d | Line number of the target	  |
 
-This predicate should specify the  shell-command   called  to invoke the
-user's editor.  The following substitutions will be made:
-
-	%e		Path name of the editor
-	%f		Path name of the file to be edited
-	%d		Line number of the target
-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 edit_command(vi,	  '%e +%d ''%f''').
 edit_command(vi,	  '%e ''%f''').
@@ -347,6 +357,13 @@ merge_locations([file(X)]-_, Loc-Spec, Loc-Spec) :-
 
 same_location(L, L, L).
 same_location([file(F1)], [file(F2)], [file(F)]) :-
+	best_same_file(F1, F2, F).
+same_location([file(F1),line(L)], [file(F2)], [file(F),line(L)]) :-
+	best_same_file(F1, F2, F).
+same_location([file(F1)], [file(F2),line(L)], [file(F),line(L)]) :-
+	best_same_file(F1, F2, F).
+
+best_same_file(F1, F2, F) :-
 	catch(same_file(F1, F2), _, fail), !,
 	atom_length(F1, L1),
 	atom_length(F2, L2),
@@ -357,7 +374,7 @@ same_location([file(F1)], [file(F2)], [file(F)]) :-
 
 merge_specs(source_file(Path), _, source_file(Path)).
 
-%	select_location(+Pairs, +UserSpec, -Location)
+%%	select_location(+Pairs, +UserSpec, -Location)
 
 do_select_location(Pairs, Spec, Location) :-
 	select_location(Pairs, Spec, Location), !, 		% HOOK
@@ -366,9 +383,6 @@ do_select_location([], Spec, _) :- !,
 	print_message(warning, edit(not_found(Spec))),
 	fail.
 do_select_location([Location-_Spec], _, Location) :- !.
-%select_location(Pairs, _, Location) :-
-%	length(Pairs, N),
-%	N > 20, !,
 do_select_location(Pairs, _, Location) :-
 	print_message(help, edit(select)),
 	list_pairs(Pairs, 0, N),
