@@ -122,9 +122,9 @@ attach_console :-
 attach_console :-
 	thread_self(Id),
 	current_prolog_flag(system_thread_id, SysId),
-	sformat(Title,
-		'SWI-Prolog Thread ~w (~d) Interactor',
-		[Id, SysId]),
+	format(atom(Title),
+	       'SWI-Prolog Thread ~w (~d) Interactor',
+	       [Id, SysId]),
 	(   current_prolog_flag(windows, true)
 	->  regkey(Id, Key),
 					% Avoid undefined predicate listing
@@ -210,7 +210,7 @@ tprofile(Thread) :-
 		      (	  reset_profiler,
 			  profiler(_, true)
 		      )),
-	format('Running profiler in thread ~w (press return to store) ...',
+	format('Running profiler in thread ~w (press RET to show results) ...',
 	       [Thread]),
 	flush_output,
 	get0(_),
