@@ -67,9 +67,11 @@ of the standard encountered in practical use.
 		 *	      GLOBALISE		*
 		 *******************************/
 
-%%	global_url(+URL, +Base, -Global)
+%%	global_url(+URL, +Base, -Global) is det.
 %	
 %	Translate a possibly relative URL  into   an  absolute  one.
+%	
+%	@error syntax_error(illegal_url) if URL is not legal.
 
 global_url(URL, BaseURL, Global) :-
 	(   is_absolute_url(URL),
@@ -85,9 +87,10 @@ global_url(URL, BaseURL, Global) :-
 		atom_concat(BaseURL, NoHash, Global)
 	    ;	atom_concat(BaseURL, URL, Global)
 	    )
-	;   parse_url(URL, BaseURL, Attributes),
-	    phrase(curl(Attributes), Chars),
+	;   parse_url(URL, BaseURL, Attributes)
+	->  phrase(curl(Attributes), Chars),
 	    atom_codes(Global, Chars)
+	;   throw(error(syntax_error(illegal_url), URL))
 	).
 
 %%	is_absolute_url(+URL)
@@ -392,9 +395,16 @@ make_path(Dir, Local, Path) :-
 	concat_atom([Dir, /, Local], Path).
 
 
+%%	absolute_url//
+%
+%	True if the input  describes  an   absolute  URL.  This means it
+%	starts with a URL schema. We demand a   schema  of length > 1 to
+%	avoid confusion with Windows drive letters.
+
 absolute_url -->
-	lwalpha(_),
-	schema_chars(_),
+	lwalpha(_First),
+	schema_chars(Rest),
+	{ Rest \== [] },
 	":", !.
 
 
@@ -423,7 +433,7 @@ digit(C, [C|T], T) :- code_type(C, digit).
 url([protocol(Schema)|Parts]) -->
 	schema(Schema),
 	":", !,
-	hier_part(Parts, P2),
+	hier_part(Schema, Parts, P2),
 	query(P2, P3),
 	fragment(P3, []).
 url([protocol(http)|Parts]) -->		% implicit HTTP
@@ -488,14 +498,19 @@ schema_extra(0'.).	% 0'
 
 
 %%	hier_part(+Schema, -Parts, ?Tail)//
-%	
-%	Extract the hierarchy part.
 
-hier_part(Parts, Tail) -->
+hier_part(file, [path(Path)|Tail], Tail) --> !,
+	"//",
+	(   win_drive_path(Path)
+	;   path_absolute(Path)
+	;   path_rootless(Path)
+	;   path_empty(Path)
+	), !.
+hier_part(_, Parts, Tail) -->
 	"//", !,
 	authority(Parts, [path(Path)|Tail]),
 	path_abempty(Path).
-hier_part([path(Path)|T], T) -->
+hier_part(_, [path(Path)|T], T) -->
 	(   path_absolute(Path)
 	;   path_rootless(Path)
 	;   path_empty(Path)
@@ -573,6 +588,18 @@ path_abempty(Path) -->
 	}.
 
 
+win_drive_path(Path) -->
+	drive_letter(C0),
+	":",
+	(   "/"
+	->  {Codes = [C0, 0':, 0'/|Chars]}
+	;   {Codes = [C0, 0':|Chars]}
+	),
+	segment_nz_chars(Chars, T0),
+	segments_chars(T0, []),
+	{ atom_codes(Path, Codes) }.
+
+
 path_absolute(Path) -->
 	"/",
 	segment_nz_chars(Chars, T0),
@@ -720,6 +747,13 @@ lwalpha(H) -->
 	  code_type(H, to_lower(C))
 	}.
 
+drive_letter(C) -->
+	[C],
+	{ C < 128,
+	  code_type(C, alpha)
+	}.
+
+
 		 /*******************************
 		 *	RESERVED CHARACTERS	*
 		 *******************************/
