@@ -39,14 +39,15 @@
 :- use_module(library(socket)).
 :- use_module(library(url)).
 :- use_module(http_header).
-:- use_module(http_chunked).
+:- use_module(http_stream).
 :- use_module(library(debug)).
 :- use_module(library(memfile)).
 :- use_module(library(lists)).
 :- use_module(dcg_basics).
 
 :- multifile
-	http_convert_data/4.		% http_read_data plugin-hook
+	http_convert_data/4,		% http_read_data plugin-hook
+	post_data_hook/3.		% http_post_data/3 hook
 
 :- dynamic
 	connection/4.			% Host:Port, ThreadId, In, Out
