@@ -68,6 +68,7 @@ for details.
 :- multifile
 	make_socket_hook/3,
 	accept_hook/2,
+	close_hook/1,
 	open_client_hook/5.
 
 %%	http_server(:Goal, +Options) is det.
@@ -86,7 +87,7 @@ for details.
 
 http_server(Goal, Options) :-
 	strip_module(Goal, Module, G),
-	select(port(Port), Options, Options1), !,
+	select_option(port(Port), Options, Options1), !,
 	after_option(Options1, Module, Options2),
 	make_socket(Port, Options2, Options3),
 	create_workers(Options3),
@@ -128,8 +129,7 @@ create_server(Goal, Port, Options) :-
 	memberchk(queue(Queue), Options),
 	atom_concat('http@', Port, Alias),
 	thread_create(accept_server(Goal, Options), _,
-		      [ detached(true),
-			local(128),
+		      [ local(128),
 			global(128),
 			trail(128),
 			alias(Alias)
@@ -157,15 +157,18 @@ http_current_server(Goal, Port) :-
 %	(one) can be used to profile the worker using tprofile/1.
 
 http_workers(Port, Workers) :-
-	current_server(Port, _, _, Queue),
+	must_be(integer, Port),
+	current_server(Port, _, _, Queue), !,
 	(   integer(Workers)
 	->  resize_pool(Queue, Workers)
 	;   findall(W, queue_worker(Queue, W), WorkerIDs),
 	    length(WorkerIDs, Workers)
 	).
+http_workers(Port, _) :-
+	existence_error(http_server, Port).
 
 
-%%	http_current_worker(?Port, ?ThreadID)
+%%	http_current_worker(?Port, ?ThreadID) is nondet.
 %	
 %	True if ThreadID is the identifier   of  a Prolog thread serving
 %	Port. This predicate is  motivated  to   allow  for  the  use of
@@ -185,7 +188,8 @@ http_current_worker(Port, ThreadID) :-
 accept_server(Goal, Options) :-
 	catch(accept_server2(Goal, Options), http_stop, true),
 	thread_self(Thread),
-	retract(current_server(_Port, _, Thread, _Queue)).
+	retract(current_server(_Port, _, Thread, _Queue)),
+	close_server_socket(Options).
 
 accept_server2(Goal, Options) :-
 	accept_hook(Goal, Options), !.
@@ -193,13 +197,29 @@ accept_server2(Goal, Options) :-
 	memberchk(tcp_socket(Socket), Options), !,
 	memberchk(queue(Queue), Options),
 	repeat,
-	  catch(tcp_accept(Socket, Client, Peer), E,
-		(   print_message(error, E),
-		    fail
-		)),
-	  debug(http(connection), 'New HTTP connection from ~p', [Peer]),
-	  thread_send_message(Queue, tcp_client(Client, Goal, Peer)),
-	fail.
+	  (   catch(tcp_accept(Socket, Client, Peer), E, true)
+	  ->  (   var(E)
+	      ->  debug(http(connection), 'New HTTP connection from ~p', [Peer]),
+		  thread_send_message(Queue, tcp_client(Client, Goal, Peer)),
+		  fail
+	      ;   E == http_stop
+	      ->  throw(http_stop)
+	      ;   print_message(error, E),
+		  fail
+	      )
+	  ;   print_message(error, goal_failed(tcp_accept(Socket, Client, Peer)))
+	  ).
+
+
+%%	close_server_socket(+Options)
+%
+%	Close the server socket.
+
+close_server_socket(Options) :-
+	close_hook(Options), !.
+close_server_socket(Options) :-
+	memberchk(tcp_socket(Socket), Options), !,
+	tcp_close_socket(Socket).
 
 
 %%	http_stop_server(+Port, +Options)
@@ -210,13 +230,13 @@ accept_server2(Goal, Options) :-
 %	@tbd	Realise non-graceful stop
 
 http_stop_server(Port, _Options) :-
-	must_be(integer, Port),
 	http_workers(Port, 0),
 	current_server(Port, _, Thread, Queue),
-	message_queue_destroy(Queue),
 	retractall(queue_options(Queue, _)),
 	thread_signal(Thread, throw(http_stop)),
-	catch(connect(localhost:Port), _, true).
+	catch(connect(localhost:Port), _, true),
+	thread_join(Thread, _),
+	message_queue_destroy(Queue).
 
 connect(Address) :-
         tcp_socket(Socket),
@@ -279,14 +299,22 @@ resize_pool(Queue, Size) :-
 
 %%	http_worker(+Options)
 %	
-%	A worker.  Workers simply wait until they are passes an accepted
-%	socket to process a client.
+%	Run HTTP worker main loop. Workers   simply  wait until they are
+%	passed an accepted socket to process  a client. After processing
+%	a request they keep processing requests from the same connection
+%	as long as =|Connection:  keep-alive|=   remains  requested from
+%	both parties. Otherwise they close the  connection and return to
+%	the worker pool.  See server_loop/6.
+%	
+%	If the message quit(Sender) is read   from the queue, the worker
+%	stops.
 
 http_worker(Options) :-
 	option(timeout(Timeout), Options, infinite),
 	option(queue(Queue), Options),
 	thread_at_exit(done_worker),
 	repeat,
+	  garbage_collect,
 	  thread_get_message(Queue, Message),
 	  (   Message = quit(Sender)
 	  ->  thread_self(Self),
