@@ -129,7 +129,7 @@ prolog_file(FileSpec, Options) -->
 	     ]),
 	undocumented(Objects, FileOptions).
 	  
-%%	file_objects(+FileSpec, -File, -Objects, -FileOptions, +Options) is det.
+%%	doc_file_objects(+FileSpec, -File, -Objects, -FileOptions, +Options) is det.
 %
 %	Extracts  relevant  information  for  FileSpec  from  the  PlDoc
 %	database.  FileOptions contains:
@@ -166,15 +166,9 @@ doc_file_objects(FileSpec, File, Objects, FileOptions, Options) :-
 
 module_info(File, [module(Module), public(Exports)|Options], Options) :-
 	current_module(Module, File), !,
-	export_list(Module, Public),
-	maplist(head_to_pi, Public, Exports).
+	export_list(Module, Exports).
 module_info(_, Options, Options).
 
-head_to_pi(M:Head, M:PI) :- !,
-	head_to_pi(Head, PI).
-head_to_pi(Head, Name/Arity) :-
-	functor(Head, Name, Arity).
-	
 
 %%	doc_hide_private(+Objs, +Public, +Options)
 %
@@ -221,9 +215,8 @@ private(Module:PI, Options) :-
 	     eq_pi(PI, PI2)
 	   ).
 private(Module:PI, _Options) :-
-	export_list(Module, Public),
-	\+ ( member(Head, Public),
-	     head_to_pi(Head, PI2),
+	export_list(Module, Exports),
+	\+ ( member(PI2, Exports),
 	     eq_pi(PI, PI2)
 	   ).
 
@@ -595,18 +588,24 @@ print_html_head(Out) :-
 tags(Tags) -->
 	html(dl(class=tags, Tags)).
 
-%%	tag(+Tag, +Value)// is det.
+%%	tag(+Tag, +Values:list)// is det.
 %
-%	Called from \tag(Name, Value) terms produced by doc_wiki.pl.
+%	Called from \tag(Name, Values) terms produced by doc_wiki.pl.
 
-tag(Tag, Value) -->
-	{   doc_tag_title(Tag, Title)
+tag(Tag, Values) -->
+	{   doc_tag_title(Tag, Title),
+	    atom_concat('keyword-', Tag, Class)
 	},
-	{   tag_class(Tag, Class)
-	->  true
-	;   Class = tag
-	},
-	html([dt(class=Class, Title), dd(Value)]).
+	html([ dt(class=Class, Title),
+	       \tag_values(Values, Class)
+	     ]).
+
+tag_values([], _) -->
+	[].
+tag_values([H|T], Class) -->
+	html(dd(class=Class, ['- '|H])),
+	tag_values(T, Class).
+
 
 %%	doc_tag_title(+Tag, -Title) is det.
 %
@@ -619,10 +618,7 @@ doc_tag_title(Tag, Tag).
 tag_title(compat, 'Compatibility').
 tag_title(tbd,    'To be done').
 tag_title(see,    'See also').
-
-tag_class(tbd, 		warn).
-tag_class(bug, 		error).
-tag_class(deprecated,	warn).
+tag_title(error,  'Errors').
 
 %%	params(+Params:list) is det.
 %
