@@ -5,7 +5,7 @@
     Author:        Jan Wielemaker
     E-mail:        wielemak@science.uva.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 2006-2007, University of Amsterdam
+    Copyright (C): 2006-2008, University of Amsterdam
 
     This file is covered by the `The Artistic License', also in use by
     Perl.  See http://www.perl.com/pub/a/language/misc/Artistic.html
@@ -18,7 +18,9 @@
 	    end_tests/1,		% +Name
 	    run_tests/0,		% Run all tests
 	    run_tests/1,		% Run named test-set
-	    load_test_files/1		% +Options
+	    load_test_files/1,		% +Options
+	    running_tests/0,		% Prints currently running test
+	    test_report/1		% +What
 	  ]).
 
 /** <module> Unit Testing
@@ -62,7 +64,7 @@ if_expansion((:- else), []) :-
 	    ;	X2 = X
 	    ),
 	    asserta(include_code(X2))
-	;   throw(error(context_error(no_if), _))
+	;   throw_error(context_error(no_if),_)
 	).
 if_expansion((:- endif), []) :-
 	retract(include_code(_)), !.
@@ -79,9 +81,12 @@ sicstus :- catch(current_prolog_flag(system_type, _), _, fail).
 
 
 :- if(swi).
+throw_error(Error_term,Impldef) :-
+	throw(error(Error_term,Impldef)).
+
 :- set_prolog_flag(generate_debug_info, false).
 :- use_module(library(option)).
-:- use_module(library(quintus), [subsumes_chk/2]).
+:- use_module(library(pairs)).
 
 current_test_flag(Name, Value) :-
 	current_prolog_flag(Name, Value).
@@ -91,6 +96,9 @@ set_test_flag(Name, Value) :-
 :- endif.
 
 :- if(sicstus).
+throw_error(Error_term,Impldef) :-
+	throw(error(Error_term,i(Impldef))). % SICStus 3 work around
+
 :- use_module(swi).			% SWI-Compatibility
 :- use_module(library(terms)).
 :- op(700, xfx, =@=).
@@ -118,11 +126,16 @@ current_test_flag(Name, Val) :-
 
 set_test_flag(Name, Val) :-
 	var(Name), !,
-	throw(error(instantiation_error, set_test_flag(Name,Val))).
+	throw_error(instantiation_error, set_test_flag(Name,Val)).
 set_test_flag( Name, Val ) :-
 	retractall(test_flag(Name,_)),
 	asserta(test_flag(Name, Val)).
 
+:- op(1150, fx, thread_local).
+
+user:term_expansion((:- thread_local(PI)), (:- dynamic(PI))) :-
+	prolog_load_context(module, plunit).
+
 :- endif.
 
 		 /*******************************
@@ -135,7 +148,8 @@ set_test_flag( Name, Val ) :-
    (   current_test_flag(test_options, _)
    ->  true
    ;   set_test_flag(test_options,
-		     [ run(make)	% run tests on make/0
+		     [ run(make),	% run tests on make/0
+		       sto(false)
 		     ])
    ).
 
@@ -151,6 +165,13 @@ set_test_flag( Name, Val ) :-
 %		When the tests are run.  Values are =manual=, =make=
 %		or make(all).
 %		
+%		* sto(+Bool)
+%		How to test whether code is subject to occurs check
+%		(STO).  If =false= (default), STO is not considered.
+%		If =true= and supported by the hosting Prolog, code
+%		is run in all supported unification mode and reported
+%		if the results are inconsistent.
+%		
 %	@tbd	Verify types	
 
 set_test_options(Options) :-
@@ -195,6 +216,7 @@ begin_tests(Unit, Options) :-
 
 :- if(swi).
 begin_tests(Unit, Name, File:Line, Options) :-
+	loading_tests, !,
 	'$set_source_module'(Context, Context),
 	Supers = [Context],
 	(   current_unit(Unit, Name, Supers, Options)
@@ -209,6 +231,9 @@ begin_tests(Unit, Name, File:Line, Options) :-
 	'$set_predicate_attribute'(Name:'unit test'/4, trace, 0),
 	discontiguous(Name:'unit body'/2),
 	asserta(loading_unit(Unit, Name, File, Old)).
+begin_tests(Unit, Name, File:_Line, _Options) :-
+	'$set_source_module'(Old, Old),
+	asserta(loading_unit(Unit, Name, File, Old)).
 
 set_import_modules(Module, Imports) :-
 	findall(I, import_module(Module, I), IL),
@@ -221,17 +246,21 @@ set_import_modules(Module, Imports) :-
 
 user:term_expansion((:- begin_tests(Set)),
 		    [ (:- begin_tests(Set)),
+		      (:- discontiguous(test/2)),
 		      (:- discontiguous('unit body'/2)),
 		      (:- discontiguous('unit test'/4))
 		    ]).
 
 begin_tests(Unit, Name, File:_Line, Options) :-
+	loading_tests, !,
 	(   current_unit(Unit, Name, Supers, Options)
 	->  true
 	;   retractall(current_unit(Unit, Name, _, _)),
 	    assert(current_unit(Unit, Name, Supers, Options))
 	),
 	asserta(loading_unit(Unit, Name, File, -)).
+begin_tests(Unit, Name, File:_Line, _Options) :-
+	asserta(loading_unit(Unit, Name, File, -)).
 
 :- endif.
 
@@ -247,10 +276,10 @@ end_tests(Unit) :-
 	(   Unit == StartUnit
 	->  once(retract(loading_unit(StartUnit, _, _, Old))),
 	    '$set_source_module'(_, Old)
-	;   throw(error(context_error(plunit_close(Unit, StartUnit)), _))
+	;   throw_error(context_error(plunit_close(Unit, StartUnit)), _)
 	).
 end_tests(Unit) :-
-	throw(error(context_error(plunit_close(Unit, -)), _)).
+	throw_error(context_error(plunit_close(Unit, -)), _).
 
 %%	make_unit_module(+Name, -ModuleName) is det.
 %%	unit_module(+Name, -ModuleName) is det.
@@ -264,8 +293,8 @@ make_unit_module(Unit, Module) :-
 	unit_module(Unit, Module),
 	(   current_module(Module),
 	    \+ current_unit(_, Module, _, _)
-	->  throw(error(permission_error(create, plunit, Unit),
-			'Existing module'))
+	->  throw_error(permission_error(create, plunit, Unit),
+			'Existing module')
 	;  true
 	).
 
@@ -310,7 +339,7 @@ expand_test(Name, Options0, Body,
 
 expand_option(Var, _) :-
 	var(Var), !,
-	throw(error(instantiation_error)).
+	throw_error(instantiation_error,_).
 expand_option(A == B, true(A==B)) :- !.
 expand_option(A = B, true(A=B)) :- !.
 expand_option(A =@= B, true(A=@=B)) :- !.
@@ -330,6 +359,10 @@ expand((test(Name) :- Body), Clauses) :- !,
 	expand_test(Name, [], Body, Clauses).
 expand((test(Name, Options) :- Body), Clauses) :- !,
 	expand_test(Name, Options, Body, Clauses).
+expand(test(Name), _) :- !,
+	throw_error(existence_error(body, test(Name)), _).
+expand(test(Name, _Options), _) :- !,
+	throw_error(existence_error(body, test(Name)), _).
 
 :- if(swi).
 :- multifile
@@ -363,8 +396,8 @@ must_be(Type, X) :-
 
 is_not(Type, X) :-
 	(   ground(X)
-	->  throw(error(type_error(Type, X), _))
-	;   throw(error(instantiation_error, _))
+	->  throw_error(type_error(Type, X), _)
+	;   throw_error(instantiation_error, _)
 	).
 :- endif.
 
@@ -383,7 +416,7 @@ verify_options([], _).
 verify_options([H|T], Pred) :-
 	(   call(Pred, H)
 	->  verify_options(T, Pred)
-	;   throw(error(domain_error(Pred, H), _))
+	;   throw_error(domain_error(Pred, H), _)
 	).
 
 
@@ -401,6 +434,7 @@ test_option(error(_)).
 test_option(all(_)).
 test_option(set(_)).
 test_option(nondet).
+test_option(fixme(_)).
 
 %%	test_option(+Option) is semidet.
 %
@@ -415,16 +449,23 @@ test_set_option(setup(X)) :-
 	must_be(callable, X).
 test_set_option(cleanup(X)) :-
 	must_be(callable, X).
+test_set_option(sto(V)) :-
+	nonvar(V), member(V, [finite_trees, rational_trees]).
 
 
 		 /*******************************
 		 *	  RUNNING TOPLEVEL	*
 		 *******************************/
 
-:- dynamic
+:- thread_local
 	passed/5,			% Unit, Test, Line, Det, Time
 	failed/4,			% Unit, Test, Line, Reason
-	blocked/4.			% Unit, Test, Line, Reason
+	blocked/4,			% Unit, Test, Line, Reason
+	sto/4,				% Unit, Test, Line, Results
+	fixme/5.			% Unit, Test, Line, Reason, Status
+
+:- dynamic
+	running/5.			% Unit, Test, Line, STO, Thread
 
 %%	run_tests is semidet.
 %%	run_tests(+TestSet) is semidet.
@@ -448,7 +489,7 @@ run_unit(Spec) :-
 	unit_from_spec(Spec, Unit, Tests, Module, UnitOptions),
 	(   option(blocked(Reason), UnitOptions)
 	->  info(plunit(blocked(unit(Unit, Reason))))
-	;   setup(Module, UnitOptions)
+	;   setup(Module, unit(Unit), UnitOptions)
 	->  info(plunit(begin(Spec))),
 	    forall((Module:'unit test'(Name, Line, Options, Body),
 		    matching_test(Name, Tests)),
@@ -464,10 +505,17 @@ run_unit(Spec) :-
 
 unit_from_spec(Unit, Unit, _, Module, Options) :-
 	atom(Unit), !,
-	current_unit(Unit, Module, _Supers, Options).
+	(   current_unit(Unit, Module, _Supers, Options)
+	->  true
+	;   throw_error(existence_error(unit_test, Unit), _)
+	).
 unit_from_spec(Unit:Tests, Unit, Tests, Module, Options) :-
 	atom(Unit), !,
-	current_unit(Unit, Module, _Supers, Options).
+	(   current_unit(Unit, Module, _Supers, Options)
+	->  true
+	;   throw_error(existence_error(unit_test, Unit), _)
+	).
+
 
 matching_test(X, X) :- !.
 matching_test(Name, Set) :-
@@ -475,9 +523,13 @@ matching_test(Name, Set) :-
 	memberchk(Name, Set).
 
 cleanup :-
+	thread_self(Me),
 	retractall(passed(_, _, _, _, _)),
 	retractall(failed(_, _, _, _)),
-	retractall(blocked(_, _, _, _)).
+	retractall(blocked(_, _, _, _)),
+	retractall(sto(_, _, _, _)),
+	retractall(fixme(_, _, _, _, _)),
+	retractall(running(_,_,_,_,Me)).
 
 
 %%	run_tests_in_files(+Files:list)	is det.
@@ -520,126 +572,254 @@ make_run_tests(Files) :-
 	;   true
 	).
 
+:- if(swi).
+
+unification_capability(sto_error_incomplete).
+% can detect some (almost all) STO runs
+unification_capability(rational_trees).
+unification_capability(finite_trees).
+
+set_unification_capability(Cap) :-
+	cap_to_flag(Cap, Flag),
+	set_prolog_flag(occurs_check, Flag).
+
+current_unification_capability(Cap) :-
+	current_prolog_flag(occurs_check, Flag),
+	cap_to_flag(Cap, Flag), !.
+
+cap_to_flag(sto_error_incomplete, error).
+cap_to_flag(rational_trees, false).
+cap_to_flag(finite_trees, true).
+
+:- else.
+:- if(sicstus).
+
+unification_capability(rational_trees).
+set_unification_capability(rational_trees).
+current_unification_capability(rational_trees).
+
+:- else.
+
+unification_capability(_) :-
+	fail.
+
+:- endif.
+:- endif.
+
 
 		 /*******************************
 		 *	   RUNNING A TEST	*
 		 *******************************/
 
 %%	run_test(+Unit, +Name, +Line, +Options, +Body) is det.
+%
+%	Run a single test.  
 
-run_test(Unit, Name, Line, Options, _Body) :-
-	option(blocked(Reason), Options), !,
+run_test(Unit, Name, Line, Options, Body) :-
+	current_test_flag(test_options, Options),
+	option(sto(false), Options, false), !,
+	run_test_6(Unit, Name, Line, Options, Body, Result),
+	report_result(Result, Options).
+run_test(Unit, Name, Line, Options, Body) :-
+	current_unit(Unit, _Module, _Supers, UnitOptions),
+	option(sto(Type), UnitOptions),
+	\+ option(sto(_), Options), !,
+	current_unification_capability(Cap0),
+	call_cleanup(run_test_cap(Unit, Name, Line, [sto(Type)|Options], Body),
+		     set_unification_capability(Cap0)).
+run_test(Unit, Name, Line, Options, Body) :-
+	current_unification_capability(Cap0),
+	call_cleanup(run_test_cap(Unit, Name, Line, Options, Body),
+		     set_unification_capability(Cap0)).
+
+run_test_cap(Unit, Name, Line, Options, Body) :-
+	(   option(sto(Type), Options)
+	->  unification_capability(Type),
+	    set_unification_capability(Type),
+	    run_test_6(Unit, Name, Line, Options, Body, Result),
+	    report_result(Result, Options)
+	;   findall(Key-(Type+Result),
+		    test_caps(Type, Unit, Name, Line, Options, Body, Result, Key),
+		    Pairs),
+	    group_pairs_by_key(Pairs, Keyed),
+	    (	Keyed == []
+	    ->	true
+	    ;	Keyed = [_-Results]
+	    ->	Results = [_Type+Result|_],
+		report_result(Result, Options)		% consistent results
+	    ;	pairs_values(Pairs, ResultByType),
+		report_result(sto(Unit, Name, Line, ResultByType), Options)
+	    )
+	).
+
+%%	test_caps(-Type, +Unit, +Name, +Line, +Options, +Body, -Result, -Key) is nondet.
+
+test_caps(Type, Unit, Name, Line, Options, Body, Result, Key) :-
+	unification_capability(Type),
+	set_unification_capability(Type),
+	begin_test(Unit, Name, Line, Type),
+	run_test_6(Unit, Name, Line, Options, Body, Result),
+	end_test(Unit, Name, Line, Type),
+	result_to_key(Result, Key),
+	Key \== setup_failed.
+
+result_to_key(blocked(_, _, _, _), blocked).
+result_to_key(failure(_, _, _, How0), failure(How1)) :-
+	( How0 = succeeded(_T) -> How1 = succeeded ; How0 = How1 ).
+result_to_key(success(_, _, _, Determinism, _), success(Determinism)).
+result_to_key(setup_failed(_,_,_), setup_failed).
+
+report_result(blocked(Unit, Name, Line, Reason), _) :- !,
 	assert(blocked(Unit, Name, Line, Reason)).
-run_test(Unit, Name, Line, Options, Body) :-
+report_result(failure(Unit, Name, Line, How), Options) :- !,
+	failure(Unit, Name, Line, How, Options).
+report_result(success(Unit, Name, Line, Determinism, Time), Options) :- !,
+	success(Unit, Name, Line, Determinism, Time, Options).
+report_result(setup_failed(_Unit, _Name, _Line), _Options).
+report_result(sto(Unit, Name, Line, ResultByType), Options) :-
+	assert(sto(Unit, Name, Line, ResultByType)),
+	print_message(error, plunit(sto(Unit, Name, Line))),
+	report_sto_results(ResultByType, Options).
+
+report_sto_results([], _).
+report_sto_results([Type+Result|T], Options) :-
+	print_message(error, plunit(sto(Type, Result))),
+	report_sto_results(T, Options).
+
+
+%%	run_test_6(+Unit, +Name, +Line, +Options, :Body, -Result) is det.
+%
+%	Result is one of:
+%	
+%		* blocked(Unit, Name, Line, Reason)
+%		* failure(Unit, Name, Line, How)
+%		* success(Unit, Name, Line, Determinism, Time)
+%		* setup_failed(Unit, Name, Line)
+
+run_test_6(Unit, Name, Line, Options, _Body,
+	   blocked(Unit, Name, Line, Reason)) :-
+	option(blocked(Reason), Options), !.
+run_test_6(Unit, Name, Line, Options, Body, Result) :-
 	option(all(Answer), Options), !,		% all(Bindings)
-	nondet_test(all(Answer), Unit, Name, Line, Options, Body).
-run_test(Unit, Name, Line, Options, Body) :-
+	nondet_test(all(Answer), Unit, Name, Line, Options, Body, Result).
+run_test_6(Unit, Name, Line, Options, Body, Result) :-
 	option(set(Answer), Options), !,		% set(Bindings)
-	nondet_test(set(Answer), Unit, Name, Line, Options, Body).
-run_test(Unit, Name, Line, Options, Body) :-
+	nondet_test(set(Answer), Unit, Name, Line, Options, Body, Result).
+run_test_6(Unit, Name, Line, Options, Body, Result) :-
 	option(fail, Options), !,			% fail
 	unit_module(Unit, Module),
-	setup(Module, Options), !,
-	statistics(runtime, [T0,_]),
-	(   catch(Module:Body, E, true)
-	->  (   var(E)
-	    ->	statistics(runtime, [T1,_]),
+	(   setup(Module, test(Unit,Name,Line), Options)
+	->  statistics(runtime, [T0,_]),
+	    (   catch(Module:Body, E, true)
+	    ->  (   var(E)
+		->  statistics(runtime, [T1,_]),
+		    Time is (T1 - T0)/1000.0,
+		    Result = failure(Unit, Name, Line, succeeded(Time)),
+		    cleanup(Module, Options)
+		;   Result = failure(Unit, Name, Line, E),
+		    cleanup(Module, Options)
+		)
+	    ;   statistics(runtime, [T1,_]),
 		Time is (T1 - T0)/1000.0,
-		failure(Unit, Name, Line, succeeded(Time), Options),
-		cleanup(Module, Options)
-	    ;	failure(Unit, Name, Line, E, Options),
+		Result = success(Unit, Name, Line, true, Time),
 		cleanup(Module, Options)
 	    )
-	;   statistics(runtime, [T1,_]),
-	    Time is (T1 - T0)/1000.0,
-	    success(Unit, Name, Line, true, Time, Options),
-	    cleanup(Module, Options)
+	;   Result = setup_failed(Unit, Name, Line)
 	).
-run_test(Unit, Name, Line, Options, Body) :-
-	option(true(Cmp), Options),
+run_test_6(Unit, Name, Line, Options, Body, Result) :-
+	option(true(Cmp), Options), !,
 	unit_module(Unit, Module),
-	setup(Module, Options), !,			% true(Binding)
-	statistics(runtime, [T0,_]),
-	(   catch(call_test(Module:Body, Det), E, true)
-	->  (   var(E)
-	    ->	statistics(runtime, [T1,_]),
-		Time is (T1 - T0)/1000.0,
-		(   catch(Cmp, _, fail)			% tbd: error
-		->  success(Unit, Name, Line, Det, Time, Options)
-		;   failure(Unit, Name, Line, wrong_answer(Cmp), Options)
-		),
-		cleanup(Module, Options)
-	    ;	failure(Unit, Name, Line, E, Options),
+	(   setup(Module, test(Unit,Name,Line), Options) % true(Binding)
+	->  statistics(runtime, [T0,_]),
+	    (   catch(call_det(Module:Body, Det), E, true)
+	    ->  (   var(E)
+		->  statistics(runtime, [T1,_]),
+		    Time is (T1 - T0)/1000.0,
+		    (   catch(Cmp, _, fail)			% tbd: error
+		    ->  Result = success(Unit, Name, Line, Det, Time)
+		    ;   Result = failure(Unit, Name, Line, wrong_answer(Cmp))
+		    ),
+		    cleanup(Module, Options)
+		;   Result = failure(Unit, Name, Line, E),
+		    cleanup(Module, Options)
+		)
+	    ;   Result = failure(Unit, Name, Line, failed),
 		cleanup(Module, Options)
 	    )
-	;   failure(Unit, Name, Line, failed, Options),
-	    cleanup(Module, Options)
+	;   Result = setup_failed(Unit, Name, Line)
 	).
-run_test(Unit, Name, Line, Options, Body) :-
+run_test_6(Unit, Name, Line, Options, Body, Result) :-
 	(   option(throws(Expect), Options)
 	->  true
 	;   option(error(ErrorExpect), Options)
 	->  Expect = error(ErrorExpect, _)
-	),
+	), !,
 	unit_module(Unit, Module),
-	setup(Module, Options), !,			% true
-	statistics(runtime, [T0,_]),
-	(   catch(Module:Body, E, true)
-	->  (   var(E)
-	    ->	failure(Unit, Name, Line, no_exception, Options),
-		cleanup(Module, Options)
-	    ;	statistics(runtime, [T1,_]),
-		Time is (T1 - T0)/1000.0,
-		(   match_error(Expect, E)
-		->  success(Unit, Name, Line, true, Time, Options)
-		;   failure(Unit, Name, Line, wrong_error(Expect, E), Options)
-		),
+	(   setup(Module, test(Unit,Name,Line), Options)
+	->  statistics(runtime, [T0,_]),
+	    (   catch(Module:Body, E, true)
+	    ->  (   var(E)
+		->  Result = failure(Unit, Name, Line, no_exception),
+		    cleanup(Module, Options)
+		;   statistics(runtime, [T1,_]),
+		    Time is (T1 - T0)/1000.0,
+		    (   match_error(Expect, E)
+		    ->  Result = success(Unit, Name, Line, true, Time)
+		    ;   Result = failure(Unit, Name, Line, wrong_error(Expect, E))
+		    ),
+		    cleanup(Module, Options)
+		)
+	    ;   Result = failure(Unit, Name, Line, failed),
 		cleanup(Module, Options)
 	    )
-	;   failure(Unit, Name, Line, failed, Options),
-	    cleanup(Module, Options)
+	;   Result = setup_failed(Unit, Name, Line)
 	).
-run_test(Unit, Name, Line, Options, Body) :-
+run_test_6(Unit, Name, Line, Options, Body, Result) :-
 	unit_module(Unit, Module),
-	setup(Module, Options), !,			% true
-	statistics(runtime, [T0,_]),
-	(   catch(call_test(Module:Body, Det), E, true)
-	->  (   var(E)
-	    ->	statistics(runtime, [T1,_]),
-		Time is (T1 - T0)/1000.0,
-		success(Unit, Name, Line, Det, Time, Options),
-		cleanup(Module, Options)
-	    ;	failure(Unit, Name, Line, E, Options),
+	(   setup(Module, test(Unit,Name,Line), Options)
+	->  statistics(runtime, [T0,_]),
+	    (   catch(call_det(Module:Body, Det), E, true)
+	    ->  (   var(E)
+		->  statistics(runtime, [T1,_]),
+		    Time is (T1 - T0)/1000.0,
+		    Result = success(Unit, Name, Line, Det, Time),
+		    cleanup(Module, Options)
+		;   Result = failure(Unit, Name, Line, E),
+		    cleanup(Module, Options)
+		)
+	    ;   Result = failure(Unit, Name, Line, failed),
 		cleanup(Module, Options)
 	    )
-	;   failure(Unit, Name, Line, failed, Options),
-	    cleanup(Module, Options)
+	;   Result = setup_failed(Unit, Name, Line)
 	).
-run_test(_Unit, _Name, _Line, _Options, _Body).
+
 	
-%%	non_det_test(+Expected, +Unit, +Name, +Line, +Options, +Body)
+%%	non_det_test(+Expected, +Unit, +Name, +Line, +Options, +Body, -Result)
 %
 %	Run tests on non-deterministic predicates.
 
-nondet_test(Expected, Unit, Name, Line, Options, Body) :-
+nondet_test(Expected, Unit, Name, Line, Options, Body, Result) :-
 	unit_module(Unit, Module),
-	setup(Module, Options), !,
 	result_vars(Expected, Vars),
 	statistics(runtime, [T0,_]),
-	(   catch(findall(Vars, Module:Body, Bindings), E, true)
-	->  (   var(E)
-	    ->	statistics(runtime, [T1,_]),
-		Time is (T1 - T0)/1000.0,
-	        (   nondet_compare(Expected, Bindings, Unit, Name, Line)
-		->  success(Unit, Name, Line, true, Time, Options)
-		;   failure(Unit, Name, Line, wrong_answer, Options)
-		),
-		cleanup(Module, Options)
-	    ;	failure(Unit, Name, Line, E, Options),
-		cleanup(Module, Options)
+	(   setup(Module, test(Unit,Name,Line), Options)
+	->  (   catch(findall(Vars, Module:Body, Bindings), E, true)
+	    ->  (   var(E)
+		->  statistics(runtime, [T1,_]),
+		    Time is (T1 - T0)/1000.0,
+		    (   nondet_compare(Expected, Bindings, Unit, Name, Line)
+		    ->  Result = success(Unit, Name, Line, true, Time)
+		    ;   Result = failure(Unit, Name, Line, wrong_answer)
+		    ),
+		    cleanup(Module, Options)
+		;   Result = failure(Unit, Name, Line, E),
+		    cleanup(Module, Options)
+		)
 	    )
+	;   Result = setup_failed(Unit, Name, Line)
 	).
 
+
 %%	result_vars(+Expected, -Vars) is det.
 %	
 %	Create a term v(V1, ...) containing all variables at the left
@@ -685,30 +865,19 @@ cmp(Var =@= Value, Var, variant, Value). % variant/2 is the same =@=
 :- endif.
 
 
-%%	call_test(:Goal, -Det) is nondet.
+%%	call_det(:Goal, -Det) is nondet.
 %
 %	True if Goal succeeded.  Det is unified to =true= if Goal left
 %	no choicepoints and =false= otherwise.
 
-:- if(swi).
-call_test(Goal, Det) :-
-	Goal,
-	deterministic(Det).
+:- if((swi|sicstus)).
+call_det(Goal, Det) :-
+	call_cleanup(Goal,Det0=true),
+	( var(Det0) -> Det = false ; Det = true ).
 :- else.
-:- if(sicstus).
-call_test(Goal, Det) :-
-	statistics(choice, [Used0|_]),
-	Goal,
-	statistics(choice, [Used1|_]),
-	(   Used1 =:= Used0
-	->  Det = true
-	;   Det = false
-	).
-:- else.
-call_test(Goal, true) :-
+call_det(Goal, true) :-
 	call(Goal).
 :- endif.
-:- endif.
 
 %%	match_error(+Expected, +Received) is semidet.
 %
@@ -718,34 +887,39 @@ call_test(Goal, true) :-
 match_error(Expect, Rec) :-
 	subsumes_chk(Expect, Rec).
 
-%%	setup(+Module, +Options) is semidet.
+%%	setup(+Module, +Context, +Options) is semidet.
 %
 %	Call the setup handler and  fail  if   it  cannot  run  for some
 %	reason. The condition handler is  similar,   but  failing is not
-%	considered an error.
+%	considered an error.  Context is one of
+%	
+%	    * unit(Unit)
+%	    If it is the setup handler for a unit
+%	    * test(Unit,Name,Line)
+%	    If it is the setup handler for a test
 
-setup(Module, Options) :-
+setup(Module, Context, Options) :-
 	option(setup(Setup), Options), !,
 	(   catch(Module:Setup, E, true)
 	->  (   var(E)
 	    ->	true
-	    ;	print_message(error, E), 	% TBD
+	    ;	print_message(error, plunit(error(setup, Context, E))),
 		fail
 	    )
 	;   print_message(error, error(goal_failed(Setup), _)),
 	    fail
 	).
-setup(Module, Options) :-
+setup(Module, Context, Options) :-
 	option(condition(Setup), Options), !,
 	(   catch(Module:Setup, E, true)
 	->  (   var(E)
 	    ->	true
-	    ;	print_message(error, E), 	% TBD
+	    ;	print_message(error, plunit(error(condition, Context, E))),
 		fail
 	    )
 	;   fail
 	).
-setup(_,_).
+setup(_,_,_).
 
 %%	cleanup(+Module, +Options) is det.
 %
@@ -762,6 +936,18 @@ cleanup(Module, Options) :-
 	;   print_message(warning, goal_failed(Cleanup, '(cleanup handler)'))
 	).
 
+success(Unit, Name, Line, Det, _Time, Options) :-
+	memberchk(fixme(Reason), Options), !,
+	(   (   Det == true
+	    ;	memberchk(nondet, Options)
+	    )
+	->  put_char(user_error, +),
+	    Ok = passed
+	;   put_char(user_error, !),
+	    Ok = nondet
+	),
+	flush_output(user_error),
+	assert(fixme(Unit, Name, Line, Reason, Ok)).
 success(Unit, Name, Line, Det, Time, Options) :-
 	assert(passed(Unit, Name, Line, Det, Time)),
 	(   (   Det == true
@@ -773,8 +959,13 @@ success(Unit, Name, Line, Det, Time, Options) :-
 	),
 	flush_output(user_error).
 
-failure(Unit, Name, Line, E, _Options) :-
-	report_failure(Unit, Name, Line, E),
+failure(Unit, Name, Line, _, Options) :-
+	memberchk(fixme(Reason), Options), !,
+	put_char(user_error, -),
+	flush_output(user_error),
+	assert(fixme(Unit, Name, Line, Reason, failed)).
+failure(Unit, Name, Line, E, Options) :-
+	report_failure(Unit, Name, Line, E, Options),
 	assert_cyclic(failed(Unit, Name, Line, E)).
 
 %%	assert_cyclic(+Term) is det.
@@ -808,18 +999,76 @@ assert_cyclic(Term) :-
 		 *	      REPORTING		*
 		 *******************************/
 
+%%	begin_test(Unit, Test, Line, STO) is det.
+%%	end_test(Unit, Test, Line, STO) is det.
+%
+%	Maintain running/5 and report a test has started/is ended using
+%	a =silent= message:
+%	
+%	    * plunit(begin(Unit:Test, File:Line, STO))
+%	    * plunit(end(Unit:Test, File:Line, STO))
+%	    
+%	@see message_hook/3 for intercepting these messages    
+
+begin_test(Unit, Test, Line, STO) :-
+	thread_self(Me),
+	assert(running(Unit, Test, Line, STO, Me)),
+	unit_file(Unit, File),
+	print_message(silent, plunit(begin(Unit:Test, File:Line, STO))).
+
+end_test(Unit, Test, Line, STO) :-
+	thread_self(Me),
+	retractall(running(_,_,_,_,Me)),
+	unit_file(Unit, File),
+	print_message(silent, plunit(end(Unit:Test, File:Line, STO))).
+
+%%	running_tests is det.
+%
+%	Print the currently running test.
+
+running_tests :-
+	running_tests(Running),
+	print_message(informational, plunit(running(Running))).
+
+running_tests(Running) :-
+	findall(running(Unit:Test, File:Line, STO, Thread),
+		(   running(Unit, Test, Line, STO, Thread),
+		    unit_file(Unit, File)
+		), Running).
+
+
 %%	report is semidet.
 %
 %	True if there are no errors.  If errors were encountered, report
 %	them to current output and fail.
 
 report :-
-	report_blocked,
-	report_failed.
+	number_of_clauses(passed/5, Passed),
+	number_of_clauses(failed/4, Failed),
+	number_of_clauses(blocked/4, Blocked),
+	number_of_clauses(sto/4, STO),
+	(   Passed+Failed+Blocked+STO =:= 0
+	->  info(plunit(no_tests))
+	;   Failed+Blocked+STO =:= 0
+	->  report_fixme,
+	    info(plunit(all_passed(Passed)))
+	;   report_blocked,
+	    report_fixme,
+	    report_failed,
+	    report_sto
+	).
+
+number_of_clauses(F/A,N) :-
+	(   current_predicate(F/A)
+	->  functor(G,F,A),
+	    findall(t, G, Ts),
+	    length(Ts, N)
+	;   N = 0
+	).
 
 report_blocked :-
-	predicate_property(blocked(_,_,_,_), number_of_clauses(N)),
-	N > 0,
+	number_of_clauses(blocked/4,N),
+	N > 0, !,
 	info(plunit(blocked(N))),
 	(   blocked(Unit, Name, Line, Reason),
 	    unit_file(Unit, File),
@@ -830,17 +1079,53 @@ report_blocked :-
 report_blocked.
 
 report_failed :-
-	predicate_property(failed(_,_,_,_), number_of_clauses(N)),
+	number_of_clauses(failed/4, N),
 	N > 0, !,
 	info(plunit(failed(N))),
 	fail.
 report_failed :-
 	info(plunit(failed(0))).
 
-report_failure(Unit, Name, Line, Error) :-
+report_sto :-
+	number_of_clauses(sto/4, N),
+	N > 0, !,
+	info(plunit(sto(N))),
+	fail.
+report_sto :-
+	info(plunit(sto(0))).
+
+report_fixme :-
+	report_fixme(_,_,_).
+
+report_fixme(TuplesF, TuplesP, TuplesN) :-
+	fixme(failed, TuplesF, Failed),
+	fixme(passed, TuplesP, Passed),
+	fixme(nondet, TuplesN, Nondet),
+	print_message(informational, plunit(fixme(Failed, Passed, Nondet))).
+
+
+fixme(How, Tuples, Count) :-
+	findall(fixme(Unit, Name, Line, Reason, How),
+		fixme(Unit, Name, Line, Reason, How), Tuples),
+	length(Tuples, Count).
+	
+
+report_failure(Unit, Name, Line, Error, _Options) :-
 	print_message(error, plunit(failed(Unit, Name, Line, Error))).
 
 
+%%	test_report(What) is det.
+%
+%	Produce reports on test results after the run.
+
+test_report(fixme) :- !,
+	report_fixme(TuplesF, TuplesP, TuplesN),
+	append([TuplesF, TuplesP, TuplesN], Tuples),
+	print_message(informational, plunit(fixme(Tuples))).
+test_report(What) :-
+	throw_error(domain_error(report_class, What), _).
+
+
 		 /*******************************
 		 *	       INFO		*
 		 *******************************/
@@ -913,14 +1198,23 @@ message_level(Level) :-
 	;   Level = silent
 	).
 
+locationprefix(File:Line) --> !,
+	[ '~w:~d:\n\t'-[File,Line]].
+locationprefix(test(Unit,_Test,Line)) --> !,
+	{ unit_file(Unit, File) },
+	locationprefix(File:Line).
+locationprefix(unit(Unit)) --> !,
+	[ 'PL-Unit: unit ~w: '-[Unit] ].
+locationprefix(FileLine) -->
+	{ throw_error(type_error(locationprefix,FileLine), _) }.
 
 message(error(context_error(plunit_close(Name, -)), _)) -->
 	[ 'PL-Unit: cannot close unit ~w: no open unit'-[Name] ].
 message(error(context_error(plunit_close(Name, Start)), _)) -->
 	[ 'PL-Unit: cannot close unit ~w: current unit is ~w'-[Name, Start] ].
 message(plunit(nondet(File, Line, Name))) -->
-	[ '~w:~d: PL-Unit: Test ~w: Test succeeded with choicepoint'-
-	  [File, Line, Name] ].
+	locationprefix(File:Line),
+	[ 'PL-Unit: Test ~w: Test succeeded with choicepoint'- [Name] ].
 					% Unit start/end
 :- if(swi).
 message(plunit(begin(Unit))) -->
@@ -935,52 +1229,195 @@ message(plunit(end(_Unit))) -->
 :- endif.
 message(plunit(blocked(unit(Unit, Reason)))) -->
 	[ 'PL-Unit: ~w blocked: ~w'-[Unit, Reason] ].
+message(plunit(running([]))) --> !,
+	[ 'PL-Unit: no tests running' ].
+message(plunit(running([One]))) --> !,
+	[ 'PL-Unit: running ' ],
+	running(One).
+message(plunit(running(More))) --> !,
+	[ 'PL-Unit: running tests:', nl ],
+	running(More).
+message(plunit(fixme([]))) --> !.
+message(plunit(fixme(Tuples))) --> !,
+	fixme_message(Tuples).
 
 					% Blocked tests
+message(plunit(blocked(1))) --> !,
+	[ 'one test is blocked:'-[] ].
 message(plunit(blocked(N))) -->
-	[ '~D tests where blocked'-[N] ].
+	[ '~D tests are blocked:'-[N] ].
 message(plunit(blocked(Pos, Name, Reason))) -->
-	[ '  ~w: test ~w: ~w'-[Pos, Name, Reason] ].
+	locationprefix(Pos),
+	[ 'test ~w: ~w'-[Name, Reason] ].
 
 					% fail/success
+message(plunit(no_tests)) --> !,
+	[ 'No tests to run' ].
+message(plunit(all_passed(Count))) --> !,
+	[ 'All ~D tests passed'-[Count] ].
 message(plunit(failed(0))) --> !,
-	[ 'All tests passed'-[] ].
+	[].
+message(plunit(failed(1))) --> !,
+	[ '1 test failed'-[] ].
 message(plunit(failed(N))) -->
 	[ '~D tests failed'-[N] ].
+message(plunit(sto(0))) --> !,
+	[].
+message(plunit(sto(N))) -->
+	[ '~D test results depend on unification mode'-[N] ].
+message(plunit(fixme(0,0,0))) -->
+	[].
+message(plunit(fixme(Failed,0,0))) --> !,
+	[ 'all ~D tests flagged FIXME failed'-[Failed] ].
+message(plunit(fixme(Failed,Passed,0))) -->
+	[ 'FIXME: ~D failed; ~D passed'-[Failed, Passed] ].
+message(plunit(fixme(Failed,Passed,Nondet))) -->
+	{ TotalPassed is Passed+Nondet },
+	[ 'FIXME: ~D failed; ~D passed; (~D nondet)'-[Failed, TotalPassed, Nondet] ].
 message(plunit(failed(Unit, Name, Line, Failure))) -->
        { unit_file(Unit, File) },
-       [ '~w:~w: test ~w: '- [File, Line, Name] ],
+       locationprefix(File:Line),
+       ['test ~w: '- [Name] ],
        failure(Failure).
+					% Setup/condition errors
+message(plunit(error(Where, Context, Exception))) -->
+	locationprefix(Context),
+	{ message_to_string(Exception, String) },
+	[ 'error in ~w: ~w'-[Where, String] ].
+
+					% STO messages
+message(plunit(sto(Unit, Name, Line))) -->
+	{ unit_file(Unit, File) },
+       locationprefix(File:Line),
+       ['test ~w is subject to occurs check (STO): '- [Name] ].
+message(plunit(sto(Type, Result))) -->
+	sto_type(Type),
+	sto_result(Result).
+
+					% Interrupts (SWI)
+:- if(swi).
+message(interrupt(begin)) -->
+	{ thread_self(Me),
+	  running(Unit, Test, Line, STO, Me), !,
+	  unit_file(Unit, File)
+	},
+	[ 'Interrupted test '-[] ],
+	running(running(Unit:Test, File:Line, STO, Me)),
+	[nl],
+	'$messages':prolog_message(interrupt(begin)).
+message(interrupt(begin)) -->
+	'$messages':prolog_message(interrupt(begin)).
+:- endif.
+
+sto_type(sto_error_incomplete) -->
+	[ 'Finite trees (error checking): ' ].
+sto_type(rational_trees) -->
+	[ 'Rational trees: ' ].
+sto_type(finite_trees) -->
+	[ 'Finite trees: ' ].
+
+sto_result(success(_Unit, _Name, _Line, Det, Time)) -->
+	det(Det),
+	[ ' success in ~2f seconds'-[Time] ].
+sto_result(failure(_Unit, _Name, _Line, How)) -->
+	failure(How).
+
+det(true) -->
+	[ 'deterministic' ].
+det(false) -->
+	[ 'non-deterministic' ].
+
+running(running(Unit:Test, File:Line, STO, Thread)) -->
+	thread(Thread),
+	[ '~q:~q at ~w:~d'-[Unit, Test, File, Line] ],
+	current_sto(STO).
+running([H|T]) -->
+	['\t'], running(H),
+	(   {T == []}
+	->  []
+	;   [nl], running(T)
+	).
+
+thread(main) --> !.
+thread(Other) -->
+	[' [~w] '-[Other] ].
+	
+current_sto(sto_error_incomplete) -->
+	[ ' (STO: error checking)' ].
+current_sto(rational_trees) -->
+	[].
+current_sto(finite_trees) -->
+	[ ' (STO: occurs check enabled)' ].
+
+:- if(swi).
+write_term(T, OPS) -->
+	['~@'-[write_term(T,OPS)]].
+:- else.
+write_term(T, _OPS) -->
+	['~q'-[T]].
+:- endif.
+
+expected_got_ops_(Ex, E, OPS, Goals) -->
+	['    Expected: '-[]], write_term(Ex, OPS), [nl],
+	['    Got:      '-[]], write_term(E,  OPS), [nl],
+	( { Goals = [] } -> []
+	; ['       with: '-[]], write_term(Goals, OPS), [nl]
+	).
 
 
+failure(Var) -->
+	{ var(Var) }, !,
+	[ 'Unknown failure?' ].
 failure(succeeded(Time)) --> !,
 	[ 'must fail but succeeded in ~2f seconds~n'-[Time] ].
 failure(wrong_error(Expected, Error)) --> !,
-	{ copy_term(Expected-Error, Ex-E),
-	  numbervars(Ex-E, 0, _, [singletons(true)]),
+	{ copy_term(Expected-Error, Ex-E, Goals),
+	  numbervars(Ex-E-Goals, 0, _),
 	  write_options(OPS)
 	},
-	[ 'wrong error', nl, 
-	  '    Expected: ~W'-[Ex, OPS], nl,
-	  '    Got:      ~W'-[E, OPS], nl
-	].
+	[ 'wrong error'-[], nl ],
+	expected_got_ops_(Ex, E, OPS, Goals).
 failure(wrong_answer(Cmp)) -->
 	{ Cmp =.. [Op,Answer,Expected], !,
-	  copy_term(Expected-Answer, Ex-A),
-	  numbervars(Ex-A, 0, _, [singletons(true)]),
+	  copy_term(Expected-Answer, Ex-A, Goals),
+	  numbervars(Ex-A-Goals, 0, _),
 	  write_options(OPS)
 	},
-	[ 'wrong answer (compared using ~w)'-[Op], nl, 
-	  '    Expected: ~W'-[Ex, OPS], nl,
-	  '    Got:      ~W'-[A, OPS], nl
-	].
+	[ 'wrong answer (compared using ~w)'-[Op], nl ],
+	expected_got_ops_(Ex, A, OPS, Goals).
+:- if(swi).
+failure(Error) -->
+	{ Error = error(_,_), !,
+	  message_to_string(Error, Message)
+	},
+	[ 'received error: ~w'-[Message] ].
+:- endif.
 failure(Why) -->
 	[ '~p~n'-[Why] ].
 
+fixme_message([]) --> [].
+fixme_message([fixme(Unit, _Name, Line, Reason, How)|T]) -->
+	{ unit_file(Unit, File) },
+	fixme_message(File:Line, Reason, How),
+	(   {T == []}
+	->  []
+	;   [nl],
+	    fixme_message(T)
+	).
+
+fixme_message(Location, Reason, failed) -->
+	[ 'FIXME: ~w: ~w'-[Location, Reason] ].
+fixme_message(Location, Reason, passed) -->
+	[ 'FIXME: ~w: passed ~w'-[Location, Reason] ].
+fixme_message(Location, Reason, nondet) -->
+	[ 'FIXME: ~w: passed (nondet) ~w'-[Location, Reason] ].
+
+
 write_options([ numbervars(true),
 		quoted(true),
 		portray(true),
-		max_depth(10)
+		max_depth(10),
+		attributes(portray)
 	      ]).
 
 :- if(swi).
@@ -1006,7 +1443,7 @@ user:generate_message_hook(Message) -->
 	message(Message),
 	[nl].				% SICStus requires nl at the end
 
-%	user:message_hook(+Sevirity, +Message, +Lines) is semidet.
+%	user:message_hook(+Severity, +Message, +Lines) is semidet.
 %
 %	Redefine printing some messages. It appears   SICStus has no way
 %	to get multiple messages at the same   line, so we roll our own.
