@@ -1,4 +1,4 @@
-/*  $Id: event.c,v 1.45 2003/10/14 10:11:28 jan Exp $
+/*  $Id$
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -41,7 +41,7 @@ static unsigned long last_time	  = 0L;
 static Int	         last_down_bts    = ZERO;
 static int	         last_down_x      = -1000; /* multiclick detection */
 static int	         last_down_y	  = -1000;
-static int	         last_down_time   = 0;
+static unsigned long     last_down_time   = 0;
 static unsigned int	 multi_click_time = 400;
 static int	         multi_click_diff = 4;
 static int	         last_click_type  = CLICK_TYPE_triple;
@@ -91,7 +91,7 @@ initialiseEvent(EventObj e, Name id, Any window,
     int px  = valInt(x);
     int py  = valInt(y);
 
-    DEBUG(NAME_multiclick, Cprintf("t: %d (%d), x: %d (%d), y: %d (%d) --> ",
+    DEBUG(NAME_multiclick, Cprintf("t: %ld (%ld), x: %d (%d), y: %d (%d) --> ",
 				   t, last_down_time, px, last_down_x,
 				   py, last_down_y));
 
@@ -163,7 +163,7 @@ considerLocStillEvent()
       return;
     if ( instanceOfObject(last_window, ClassWindow) &&
 	 !onFlag(last_window, F_FREED|F_FREEING) &&
-	 last_x > 0 && last_y > 0 )
+	 valInt(last_x) > 0 && valInt(last_y) > 0 )
     { ServiceMode(is_service_window(last_window),
 		  { AnswerMark mark;
 		    EventObj e;
@@ -560,7 +560,7 @@ status
 get_xy_event(EventObj ev, Any obj, Bool area, Int *rx, Int *ry)
 { int x = 0, y = 0;
 
-  if ( isNil(ev->window) )
+  if ( isNil(ev->window) || onFlag(ev->window, F_FREEING|F_FREED) )
   { *rx = ev->x;
     *ry = ev->y;
     succeed;
