@@ -1,11 +1,11 @@
-/*  $Id: self.c,v 1.74 2005/11/21 19:38:59 jan Exp $
+/*  $Id$
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
     Author:        Jan Wielemaker and Anjo Anjewierden
-    E-mail:        jan@swi.psy.uva.nl
+    E-mail:        wielemak@science.uva.nl
     WWW:           http://www.swi.psy.uva.nl/projects/xpce/
-    Copyright (C): 1985-2002, University of Amsterdam
+    Copyright (C): 1985-2007, University of Amsterdam
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -22,7 +22,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
-#if !defined(WIN32) && !defined(__WIN32__)
+#if !defined(__WINDOWS__) && !defined(__WINDOWS__)
 #include <version.h>			/* get MACHINE and PCE_VERSION */
 #endif
 #include <h/kernel.h>
@@ -39,7 +39,7 @@
 #include <h/unix.h>
 #include <errno.h>
 
-#ifdef __WIN32__
+#ifdef __WINDOWS__
 #undef MACHINE
 #define MACHINE "i386"
 #undef OS
@@ -60,6 +60,23 @@ static void	run_pce_atexit_hooks(void);
 #endif
 #endif
 
+static int
+setAppDataPce(Pce pce)
+{ Name appdataname;
+
+#ifdef __WINDOWS__
+  if ( !(appdataname = ws_appdata("xpce")) )
+    appdataname = CtoName("~/.xpce");
+#else
+  appdataname = CtoName("~/.xpce");
+#endif
+  assign(pce, application_data, newObject(ClassDirectory, appdataname, EAV));
+
+  succeed;
+}
+
+
+
 /* The MacOS X hack.  The mac loader doesn't want to load ker/glob.o, from
    libXPCE.a as it only contains common variables.  This is fixed by adding
    a function and calling it from here.  See __APPLE__ below.
@@ -79,6 +96,8 @@ initialisePce(Pce pce)
  }
 #endif
 
+  assert( sizeof(string) == 2*sizeof(void*) );
+
 #ifndef O_RUNTIME
   assign(pce, debugging,              OFF);
   assign(pce, trap_errors,	      ON);
@@ -91,6 +110,7 @@ initialisePce(Pce pce)
 
   assign(pce, home,		      DEFAULT);
   assign(pce, defaults,		      CtoString("$PCEHOME/Defaults"));
+  setAppDataPce(pce);
   assign(pce, version,                CtoName(PCE_VERSION));
   assign(pce, machine,                CtoName(MACHINE));
   assign(pce, operating_system,       CtoName(OS));
@@ -199,7 +219,7 @@ Name
 getOsErrorPce(Pce pce)
 {
 #if HAVE_STRERROR
-#ifdef __WIN32__
+#ifdef __WINDOWS__
   return CtoName(strerror(_xos_errno()));
 #else
   return CtoName(strerror(errno));
@@ -300,18 +320,26 @@ getHomePce(Pce pce)
 }
 
 
+#ifdef __WINDOWS__
+#define vareq(s, q) (stricmp(s, q) == 0)
+#else
+#define vareq(s, q) (strcmp(s,q) == 0)
+#endif
+
 Name
 getEnvironmentVariablePce(Pce pce, Name name)
 { char *s;
 
   if ( (s = getenv(strName(name))) )
     answer(CtoName(s));
-#if defined(WIN32)			/* case-insensitive files */
-  if ( stricmp(strName(name), "PCEHOME") == 0 )
-#else
-  if ( streq(strName(name), "PCEHOME") )
-#endif
+  if ( vareq(strName(name), "PCEHOME") )
     answer(get(PCE, NAME_home, EAV));
+  if ( vareq(strName(name), "PCEAPPDATA") )
+  { Directory dir = get(PCE, NAME_applicationData, EAV);
+
+    if ( dir )
+      return get(dir, NAME_path, EAV);
+  }
 
   fail;
 }
@@ -574,8 +602,12 @@ static status
 bannerPce(Pce pce)
 { Name host = get(HostObject(), NAME_system, EAV);
 
-#ifdef __WIN32__
+#ifdef __WINDOWS__
+#ifdef WIN64
+  writef("XPCE %s for %I%IWin64: XP 64-bit edition%I%I\n",
+#else
   writef("XPCE %s for %I%IWin32: NT,2000,XP%I%I\n",
+#endif
 #else
   writef("XPCE %s for %s-%s and X%dR%d\n",
 #endif
@@ -584,7 +616,7 @@ bannerPce(Pce pce)
 	 pce->operating_system,
 	 pce->window_system_version,
 	 pce->window_system_revision);
-  writef("Copyright (C) 1993-2005 University of Amsterdam.\n"
+  writef("Copyright (C) 1993-2007 University of Amsterdam.\n"
 	 "XPCE comes with ABSOLUTELY NO WARRANTY. "
 	 "This is free software,\nand you are welcome to redistribute it "
 	 "under certain conditions.\n");
@@ -619,7 +651,7 @@ infoPce(Pce pce)
   writef("	Release:            %s\n", pce->version);
   writef("	System:             %s\n", pce->machine);
   writef("	Operating System:   %s\n", pce->operating_system);
-#ifdef WIN32				/* TBD: Cygwin? */
+#ifdef __WINDOWS__				/* TBD: Cygwin? */
   writef("	Window System:      windows %s.%s\n",
 	 pce->window_system_version,
 	 pce->window_system_revision);
@@ -1221,6 +1253,8 @@ static vardecl var_pce[] =
      NAME_environment, "PCE's home directory"),
   IV(NAME_defaults, "source_sink|char_array", IV_BOTH,
      NAME_environment, "File/rc from which to load defaults"),
+  IV(NAME_applicationData, "directory", IV_BOTH,
+     NAME_environment, "Directory for application data"),
   IV(NAME_version, "name", IV_NONE,
      NAME_version, "Version indication"),
   IV(NAME_machine, "name", IV_GET,
@@ -1390,7 +1424,7 @@ static classvardecl rc_pce[] =
 	   "				      '3d', grey70),\n"
 	   "    _mark_elevation   @= elevation(mark, 0),\n"
 	   "    _win_pen	  @= number(0))",
-	   /*WIN32*/
+	   /*__WINDOWS__*/
            "and(_dialog_bg     @= colour(win_btnface),\n"
 	   "    _graph_bg      @= colour(win_window),\n"
 	   "    _win_pen       @= number(1),\n"
