@@ -1,4 +1,4 @@
-/*  $Id: frame.c,v 1.66 2006/04/13 11:47:55 jan Exp $
+/*  $Id$
 
     Part of XPCE --- The SWI-Prolog GUI toolkit
 
@@ -244,7 +244,7 @@ openFrame(FrameObj fr, Point pos, Bool grab, Bool normalise)
   { x = pos->x;
     y = pos->y;
 
-#ifdef WIN32
+#ifdef __WINDOWS__
   setpos:
 #endif
     if ( normalise == ON )
@@ -278,7 +278,7 @@ openFrame(FrameObj fr, Point pos, Bool grab, Bool normalise)
   } else if ( notNil(fr->geometry) )
   { ws_x_geometry_frame(fr, fr->geometry, DEFAULT);
   }					
-#ifdef WIN32				/* But in Windows `do-it-yourself' */
+#ifdef __WINDOWS__				/* But in Windows `do-it-yourself' */
   else if ( notNil(fr->transient_for) )
   { Area pa = fr->transient_for->area;
     int xb, yb, ycap;
@@ -614,12 +614,6 @@ hiddenFrame(FrameObj fr)
 		*       AREA MANAGEMENENT	*
 		********************************/
 
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Get XY coordinate of frame if  its  center   must  be  at pos. If Pos is
-DEFAULT it is centered in the given   monitor. If the monitor is default
-too, we deduce the most recent monitor from the event.
-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-
 static Monitor
 CurrentMonitor(FrameObj fr)
 { DisplayObj d = fr->display;
@@ -630,8 +624,15 @@ CurrentMonitor(FrameObj fr)
       return mon;
   } else if ( notNil(d) && instanceOfObject(EVENT->value, ClassEvent) )
   { EventObj ev = EVENT->value;
+    Point pt;
 
-    if ( (mon=getMonitorDisplay(d, getPositionEvent(ev, d))) )
+    if ( notNil(ev->window) && offFlag(ev->window, F_FREEING|F_FREED) )
+    { pt = getPositionEvent(ev, d);
+    } else
+    { pt = getPointerLocationDisplay(d);
+    }
+
+    if ( pt && (mon=getMonitorDisplay(d, pt)) )
       return mon;
   }
 
@@ -645,6 +646,12 @@ CurrentMonitor(FrameObj fr)
 }
 
 
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+Get XY coordinate of frame if  its  center   must  be  at pos. If Pos is
+DEFAULT it is centered in the given   monitor. If the monitor is default
+too, we deduce the most recent monitor from the event.
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
 static int
 get_position_from_center_frame(FrameObj fr, Monitor mon, Point pos,
 			       int *x, int *y)
@@ -784,7 +791,7 @@ getGeometryFrame(FrameObj fr)
     { Int n = getIndexChain(fr->display->monitors, mon);
 
       if ( n )
-	sprintf(buf+strlen(buf), "@%ld", valInt(n)-1);
+	sprintf(buf+strlen(buf), "@" INTPTR_FORMAT, valInt(n)-1);
     }
 
     answer(CtoName(buf));
