@@ -95,12 +95,12 @@ isnl(wint_t chr)
 
 static int
 isperiod(wint_t chr)
-{ return strchr(".?!", chr) != NULL;
+{ return chr && strchr(".?!", chr) != NULL;
 }
 
 static int
 isquote(wint_t chr)
-{ return strchr("'`\"", chr) != NULL;
+{ return chr && strchr("'`\"", chr) != NULL;
 }
 
 static int
@@ -308,9 +308,13 @@ do_char_type(term_t chr, term_t class, control_t h, int how)
       if ( do_enum == ENUM_BOTH )
 	return PL_error("char_type", 2, NULL, ERR_INSTANTIATION);
 
-      if ( !(do_enum & ENUM_CHAR) &&
-	   !PL_get_char(chr, &c, FALSE) )
-	fail;
+      if ( !(do_enum & ENUM_CHAR) )
+      { if ( !PL_get_char(chr, &c, TRUE) )
+	  fail;
+	if ( c == -1 )
+	  return PL_unify_atom(class, ATOM_end_of_file);
+      }
+
       if ( !(do_enum & ENUM_CLASS) )
       { if ( !PL_get_name_arity(class, &cn, &arity) ||
 	     !(cc = char_type_by_name(cn, arity)) )
@@ -456,6 +460,34 @@ PRED_IMPL("iswctype", 2, iswctype, 0)
 #endif
 
 
+static int
+init_tout(PL_chars_t *t, size_t len)
+{ switch(t->encoding)
+  { case ENC_ISO_LATIN_1:
+      if ( len < sizeof(t->buf) )
+      { t->text.t = t->buf;
+	t->storage = PL_CHARS_LOCAL;
+      } else
+      { t->text.t = PL_malloc(len);
+	t->storage = PL_CHARS_MALLOC;
+      }
+      succeed;
+    case ENC_WCHAR:
+      if ( len*sizeof(pl_wchar_t) < sizeof(t->buf) )
+      { t->text.w = (pl_wchar_t*)t->buf;
+	t->storage = PL_CHARS_LOCAL;
+      } else
+      { t->text.w = PL_malloc(len*sizeof(pl_wchar_t));
+	t->storage = PL_CHARS_MALLOC;
+      }
+      succeed;
+    default:
+      assert(0);
+      fail;
+  }
+}
+
+
 static inline wint_t
 get_chr_from_text(const PL_chars_t *t, int index)
 { switch(t->encoding)
@@ -504,17 +536,11 @@ modify_case_atom(term_t in, term_t out, int down)
     tout.length    = tin.length;
     tout.canonical = FALSE;		/* or TRUE? Can WCHAR map to ISO? */
 
+    init_tout(&tout, tin.length);
+
     if ( tin.encoding == ENC_ISO_LATIN_1 )
     { const unsigned char *in = (const unsigned char*)tin.text.t;
 
-      if ( tin.length < sizeof(tout.buf) )
-      { tout.text.t = tout.buf;
-	tout.storage = PL_CHARS_LOCAL;
-      } else
-      { tout.text.t = PL_malloc(tin.length);
-	tout.storage = PL_CHARS_MALLOC;
-      }
-
       if ( down )
       { for(i=0; i<tin.length; i++)
 	{ wint_t c = towlower(in[i]);
@@ -545,15 +571,7 @@ modify_case_atom(term_t in, term_t out, int down)
 	}
       } 
     } else
-    { if ( tin.length*sizeof(pl_wchar_t) < sizeof(tout.buf) )
-      { tout.text.w = (pl_wchar_t*)tout.buf;
-	tout.storage = PL_CHARS_LOCAL;
-      } else
-      { tout.text.w = PL_malloc(tin.length*sizeof(pl_wchar_t));
-	tout.storage = PL_CHARS_MALLOC;
-      }
-
-      if ( down )
+    { if ( down )
       { for(i=0; i<tin.length; i++)
 	{ tout.text.w[i] = towlower(tin.text.w[i]);
 	}
@@ -574,17 +592,78 @@ modify_case_atom(term_t in, term_t out, int down)
 }
 
 
-foreign_t
-pl_downcase_atom(term_t in, term_t out)
-{ return modify_case_atom(in, out, TRUE);
+static
+PRED_IMPL("downcase_atom", 2, downcase_atom, 0)
+{ return modify_case_atom(A1, A2, TRUE);
 }
 
 
-foreign_t
-pl_upcase_atom(term_t in, term_t out)
-{ return modify_case_atom(in, out, FALSE);
+static
+PRED_IMPL("upcase_atom", 2, upcase_atom, 0)
+{ return modify_case_atom(A1, A2, FALSE);
 }
 
+
+		 /*******************************
+		 *	    WHITE SPACE		*
+		 *******************************/
+
+static int
+write_normalize_space(IOSTREAM *out, term_t in)
+{ PL_chars_t tin;
+  int i, end;
+
+  if ( !PL_get_text(in, &tin, CVT_ATOMIC|CVT_EXCEPTION) )
+    return FALSE;
+
+  end = tin.length;
+  i = 0;
+
+  while(i<end && unicode_separator(get_chr_from_text(&tin, i)))
+    i++;
+  while( i<end )
+  { wint_t c;
+
+    while(i<end && !unicode_separator((c=get_chr_from_text(&tin, i))))
+    { if ( Sputcode(c, out) < 0 )
+	fail;
+      i++;
+    }
+    while(i<end && unicode_separator(get_chr_from_text(&tin, i)))
+      i++;
+    if ( i < end )
+    { if (  Sputcode(' ', out) < 0 )
+	fail;
+    }
+  }
+
+  succeed;
+}
+
+
+static
+PRED_IMPL("normalize_space", 2, normalize_space, 0)
+{ redir_context ctx;
+  word rc;
+
+  EXCEPTION_GUARDED(/*code*/
+		    if ( setupOutputRedirect(A1, &ctx, FALSE) )
+		    { if ( (rc = write_normalize_space(ctx.stream, A2)) )
+			rc = closeOutputRedirect(&ctx);
+		      else
+			discardOutputRedirect(&ctx);
+		    } else
+		      rc = FALSE;
+		    /*cleanup*/,
+		    DEBUG(1, Sdprintf("Cleanup after throw()\n"));
+		    discardOutputRedirect(&ctx);
+		    rc = PL_rethrow(););
+
+  return rc;
+}
+
+
+
 		 /*******************************
 		 *	       LOCALE		*
 		 *******************************/
@@ -597,12 +676,19 @@ Note: on some installations, locale doesn't   work correctly. Printing a
 message isn't really cute. It would be better to use printMessage(), but
 the system isn't yet initialised far enough.   Maybe we should store the
 failure and print a message at the end of the initialisation?
+
+We only return FALSE if LC_CTYPE  fails.   This  is a serious indication
+that locale support is broken. We don't   depend too much on the others,
+so we ignore possible problems.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-static void
+static int
 initLocale()
-{ if ( !setlocale(LC_CTYPE, "") )
-  { DEBUG(0, Sdprintf("Failed to set LC_CTYPE locale\n"));
+{ int rc = TRUE;
+
+  if ( !setlocale(LC_CTYPE, "") )
+  { rc = FALSE;
+    DEBUG(0, Sdprintf("Failed to set LC_CTYPE locale\n"));
   }
   if ( !setlocale(LC_TIME, "") )
   { DEBUG(0, Sdprintf("Failed to set LC_TIME locale\n"));
@@ -610,6 +696,8 @@ initLocale()
   if ( !setlocale(LC_COLLATE, "") )
   { DEBUG(0, Sdprintf("Failed to set LC_COLLATE locale\n"));
   }
+
+  return rc;
 }
 
 typedef struct
@@ -667,7 +755,7 @@ PRED_IMPL("setlocale", 3, setlocale, 0)
 
 #else
 
-#define initLocale()
+#define initLocale() 1
 
 static
 PRED_IMPL("setlocale", 3, setlocale, 0)
@@ -685,6 +773,9 @@ BeginPredDefs(ctype)
   PRED_DEF("char_type", 2, char_type, PL_FA_NONDETERMINISTIC)
   PRED_DEF("code_type", 2, code_type, PL_FA_NONDETERMINISTIC)
   PRED_DEF("setlocale", 3, setlocale, 0)
+  PRED_DEF("downcase_atom", 2, downcase_atom, 0)
+  PRED_DEF("upcase_atom", 2, upcase_atom, 0)
+  PRED_DEF("normalize_space", 2, normalize_space, 0)
 EndPredDefs
 
 
@@ -744,21 +835,24 @@ initEncoding()
   { if ( !LD->encoding )
     { char *enc;
 
-      initLocale();
-      enc = setlocale(LC_CTYPE, NULL);
+      if ( !initLocale() )
+      { LD->encoding = ENC_ISO_LATIN_1;
+      } else if ( (enc = setlocale(LC_CTYPE, NULL)) )
+      { LD->encoding = ENC_ANSI;		/* text encoding */
 
-      LD->encoding = ENC_ANSI;		/* text encoding */
+	if ( (enc = strchr(enc, '.')) )
+	{ const enc_map *m;
+	  enc++;				/* skip '.' */
 
-      if ( enc && (enc = strchr(enc, '.')) )
-      { const enc_map *m;
-	enc++;				/* skip '.' */
-
-	for ( m=map; m->name; m++ )
-	{ if ( strcmp(enc, m->name) == 0 )
-	  { LD->encoding = m->encoding;
-	    break;
+	  for ( m=map; m->name; m++ )
+	  { if ( strcmp(enc, m->name) == 0 )
+	    { LD->encoding = m->encoding;
+	      break;
+	    }
 	  }
 	}
+      } else
+      { LD->encoding = ENC_ISO_LATIN_1;
       }
     }
 
