@@ -578,6 +578,8 @@ getOutputStream(term_t t, IOSTREAM **stream)
   } else if ( PL_get_atom(t, &a) && a == ATOM_user )
   { *stream = getStream(Suser_output);
     return TRUE;
+  } else
+  { *stream = NULL;			/* make compiler happy */
   }
 
   if ( !PL_get_stream_handle(t, &s) )
@@ -605,6 +607,8 @@ getInputStream__LD(term_t t, IOSTREAM **stream ARG_LD)
   } else if ( PL_get_atom(t, &a) && a == ATOM_user )
   { *stream = getStream(Suser_input);
     return TRUE;
+  } else
+  { *stream = NULL;			/* make compiler happy */
   }
 
   if ( !get_stream_handle(t, &s, SH_ERRORS|SH_ALIAS) )
@@ -796,32 +800,23 @@ closeFiles(int all)
 }
 
 
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-Get the open OS filedescriptors, so we can close them (see System())
-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
-
-int
-openFileDescriptors(unsigned char *buf, int size)
+void
+PL_cleanup_fork(void)
 { TableEnum e;
   Symbol symb;
   int n = 0;
 
-  LOCK();
   e = newTableEnum(streamContext);
   while( (symb=advanceTableEnum(e)) )
   { IOSTREAM *s = symb->name;
     int fd;
 
-    if ( (fd=Sfileno(s)) >= 0 )
-    { if ( n > size )
-	break;
-      buf[n++] = fd;
-    }
+    if ( (fd=Sfileno(s)) >= 3 )
+      close(fd);
   }
   freeTableEnum(e);
-  UNLOCK();
 
-  return n;
+  stopItimer();
 }
 
 
@@ -1075,6 +1070,10 @@ static int
 Sgetcode_intr(IOSTREAM *s, int signals)
 { int c;
 
+#ifdef __WINDOWS__
+  int newline = s->newline;
+  s->newline = SIO_NL_POSIX;		/* avoid blocking \r */
+#endif
   do
   { Sclearerr(s);
     c = Sgetcode(s);
@@ -1082,6 +1081,9 @@ Sgetcode_intr(IOSTREAM *s, int signals)
 	    errno == EINTR &&
 	    (!signals || PL_handle_signals() >= 0)
 	  );
+#ifdef __WINDOWS__
+  s->newline = newline;
+#endif
 
   return c;
 }
@@ -2207,7 +2209,9 @@ pl_tab2(term_t out, term_t spaces)
   if ( rval )
     return streamStatus(s);
 
+#ifdef O_GMP
 error:
+#endif
   (void)streamStatus(s);
   fail;
 }
@@ -2698,7 +2702,12 @@ pl_open_null_stream(term_t stream)
 { int sflags = SIO_NBUF|SIO_RECORDPOS|SIO_OUTPUT;
   IOSTREAM *s = Snew((void *)NULL, sflags, (IOFUNCTIONS *)&nullFunctions);
 
-  return PL_unify_stream_or_alias(stream, s);
+  if ( s )
+  { s->encoding = ENC_UTF8;
+    return PL_unify_stream_or_alias(stream, s);
+  }
+
+  fail;
 }
 
 
@@ -4281,6 +4290,7 @@ pl_copy_stream_data3(term_t in, term_t out, term_t len)
 { GET_LD
   IOSTREAM *i, *o;
   int c;
+  int count = 0;
 
   if ( !getInputStream(in, &i) )
     return FALSE;
@@ -4291,7 +4301,12 @@ pl_copy_stream_data3(term_t in, term_t out, term_t len)
 
   if ( !len )
   { while ( (c = Sgetcode(i)) != EOF )
-    { if ( Sputcode(c, o) < 0 )
+    { if ( (++count % 4096) == 0 && PL_handle_signals() < 0 )
+      { releaseStream(i);
+	releaseStream(o);
+	fail;
+      }
+      if ( Sputcode(c, o) < 0 )
       { releaseStream(i);
 	return streamStatus(o);
       }
@@ -4303,7 +4318,12 @@ pl_copy_stream_data3(term_t in, term_t out, term_t len)
       fail;
     
     while ( n-- > 0 && (c = Sgetcode(i)) != EOF )
-    { if ( Sputcode(c, o) < 0 )
+    { if ( (++count % 4096) == 0 && PL_handle_signals() < 0 )
+      { releaseStream(i);
+	releaseStream(o);
+	fail;
+      }
+      if ( Sputcode(c, o) < 0 )
       { releaseStream(i);
 	return streamStatus(o);
       }
@@ -4327,24 +4347,24 @@ pl_copy_stream_data2(term_t in, term_t out)
 BeginPredDefs(file)
   PRED_DEF("set_prolog_IO", 3, set_prolog_IO, 0)
   PRED_DEF("read_pending_input", 3, read_pending_input, 0)
-  PRED_DEF("get_code", 2, get_code2, 0)
-  PRED_DEF("get_code", 1, get_code1, 0)
-  PRED_DEF("get_char", 2, get_char2, 0)
-  PRED_DEF("get_char", 1, get_char1, 0)
-  PRED_DEF("get_byte", 2, get_byte2, 0)
-  PRED_DEF("get_byte", 1, get_byte1, 0)
-  PRED_DEF("peek_code", 2, peek_code2, 0)
-  PRED_DEF("peek_code", 1, peek_code1, 0)
-  PRED_DEF("peek_char", 2, peek_char2, 0)
-  PRED_DEF("peek_char", 1, peek_char1, 0)
-  PRED_DEF("peek_byte", 2, peek_byte2, 0)
-  PRED_DEF("peek_byte", 1, peek_byte1, 0)
-  PRED_DEF("put_byte", 2, put_byte2, 0)
-  PRED_DEF("put_byte", 1, put_byte1, 0)
-  PRED_DEF("put_code", 2, put_code2, 0)
-  PRED_DEF("put_code", 1, put_code1, 0)
-  PRED_DEF("put_char", 2, put_code2, 0)
-  PRED_DEF("put_char", 1, put_code1, 0)
+  PRED_DEF("get_code", 2, get_code2, PL_FA_ISO)
+  PRED_DEF("get_code", 1, get_code1, PL_FA_ISO)
+  PRED_DEF("get_char", 2, get_char2, PL_FA_ISO)
+  PRED_DEF("get_char", 1, get_char1, PL_FA_ISO)
+  PRED_DEF("get_byte", 2, get_byte2, PL_FA_ISO)
+  PRED_DEF("get_byte", 1, get_byte1, PL_FA_ISO)
+  PRED_DEF("peek_code", 2, peek_code2, PL_FA_ISO)
+  PRED_DEF("peek_code", 1, peek_code1, PL_FA_ISO)
+  PRED_DEF("peek_char", 2, peek_char2, PL_FA_ISO)
+  PRED_DEF("peek_char", 1, peek_char1, PL_FA_ISO)
+  PRED_DEF("peek_byte", 2, peek_byte2, PL_FA_ISO)
+  PRED_DEF("peek_byte", 1, peek_byte1, PL_FA_ISO)
+  PRED_DEF("put_byte", 2, put_byte2, PL_FA_ISO)
+  PRED_DEF("put_byte", 1, put_byte1, PL_FA_ISO)
+  PRED_DEF("put_code", 2, put_code2, PL_FA_ISO)
+  PRED_DEF("put_code", 1, put_code1, PL_FA_ISO)
+  PRED_DEF("put_char", 2, put_code2, PL_FA_ISO)
+  PRED_DEF("put_char", 1, put_code1, PL_FA_ISO)
   PRED_DEF("put", 2, put2, 0)
   PRED_DEF("put", 1, put1, 0)
   PRED_DEF("skip", 1, skip1, 0)
