@@ -1123,7 +1123,8 @@ PL_get_list_chars(term_t l, char **s, unsigned flags)
 
 int
 PL_get_wchars(term_t l, size_t *length, pl_wchar_t **s, unsigned flags)
-{ PL_chars_t text;
+{ GET_LD
+  PL_chars_t text;
 
   if ( !PL_get_text(l, &text, flags) )
     return FALSE;
@@ -1141,7 +1142,8 @@ PL_get_wchars(term_t l, size_t *length, pl_wchar_t **s, unsigned flags)
 
 int
 PL_get_nchars(term_t l, size_t *length, char **s, unsigned flags)
-{ PL_chars_t text;
+{ GET_LD
+  PL_chars_t text;
 
   if ( !PL_get_text(l, &text, flags) )
     return FALSE;
@@ -2019,7 +2021,13 @@ PL_put_int64(term_t t, int64_t i)
 
 void
 _PL_put_number__LD(term_t t, Number n ARG_LD)
-{ setHandle(t, put_number(n));
+{ word w;
+
+  if ( (w=put_number(n)) )
+  { setHandle(t, w);
+  } else
+  { assert(0);
+  }
 }
 
 
@@ -2332,11 +2340,13 @@ flags: bitwise or of type and representation
 
 	Types:		PL_ATOM, PL_STRING, PL_CODE_LIST, PL_CHAR_LIST
 	Representation: REP_ISO_LATIN_1, REP_UTF8, REP_MB
+	Extra:		PL_DIFF_LIST
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 int
 PL_unify_chars(term_t t, int flags, size_t len, const char *s)
 { PL_chars_t text;
+  term_t tail;
   int rc;
 
   if ( len == (size_t)-1 )
@@ -2351,7 +2361,14 @@ PL_unify_chars(term_t t, int flags, size_t len, const char *s)
 
   flags &= ~(REP_UTF8|REP_MB|REP_ISO_LATIN_1);
 
-  rc = PL_unify_text(t, 0, &text, flags);
+  if ( (flags & PL_DIFF_LIST) )
+  { tail = t+1;
+    flags &= (~PL_DIFF_LIST);
+  } else
+  { tail = 0;
+  }
+
+  rc = PL_unify_text(t, tail, &text, flags);
   PL_free_text(&text);
 
   return rc;
@@ -2381,6 +2398,39 @@ PL_unify_int64(term_t t, int64_t i)
 }
 
 
+int
+PL_unify_int64_ex__LD(term_t t, int64_t i ARG_LD)
+{ word w = consInt(i);
+
+  if ( valInt(w) == i )
+  { Word p = valHandleP(t);
+
+    deRef(p);
+    if ( *p == w )
+      succeed;
+    if ( canBind(*p) )
+    { bindConst(p, w);
+      succeed;
+    }
+    if ( isInteger(*p) )
+      fail;
+  } else
+  { Word p = valHandleP(t);
+
+    deRef(p);
+    if ( canBind(*p) )
+    { w = globalLong(i PASS_LD);
+      bindConst(p, w);
+      succeed;
+    }
+    if ( isInteger(*p) )
+      return valInt(*p) == i ? TRUE : FALSE;
+  }
+  
+  return PL_error(NULL, 0, NULL, ERR_TYPE, ATOM_integer, t);
+}
+
+
 int
 PL_unify_pointer__LD(term_t t, void *ptr ARG_LD)
 { word w = makeNum(pointerToLong(ptr));
