@@ -62,7 +62,6 @@ COMMON(word) 		consPtr(void *p, word ts);
 #endif
 COMMON(char *) 		store_string(const char *s);
 COMMON(void) 		remove_string(char *s);
-COMMON(int) 		unboundStringHashValue(const char *t, size_t l);
 #ifndef xmalloc
 COMMON(void *) 		xmalloc(size_t size);
 COMMON(void *) 		xrealloc(void *mem, size_t size);
@@ -129,10 +128,10 @@ COMMON(void)		pushArithStack(Number n ARG_LD);
 COMMON(void)		resetArithStack(ARG1_LD);
 COMMON(Number)		argvArithStack(int n ARG_LD);
 COMMON(void)		popArgvArithStack(int n ARG_LD);
+COMMON(int)		ar_sign_i(Number n1);
 
 /* pl-bag.c */
-COMMON(void) 		resetBags(void);
-COMMON(int)  		checkBags(void);
+COMMON(void)		markAtomsFindall(PL_local_data_t *ld);
 
 /* pl-comp.c */
 COMMON(void) 		initWamTable(void);
@@ -173,6 +172,7 @@ COMMON(code) 		replacedBreak(Code PC);
 COMMON(void) 		clearBreakPointsClause(Clause clause);
 COMMON(word) 		pl_current_break(term_t ref, term_t pc, control_t h);
 COMMON(int) 		get_clause_ptr_ex(term_t ref, Clause *cl);
+COMMON(int)		unify_functor(term_t t, functor_t fd, int how);
 
 /* pl-dump.c */
 COMMON(word) 		saveProgram(term_t new);
@@ -238,6 +238,7 @@ COMMON(int) 		PL_get_list_ex(term_t l, term_t h, term_t t);
 COMMON(int) 		PL_get_nil_ex(term_t l);
 COMMON(int) 		PL_unify_bool_ex(term_t t, bool val);
 COMMON(int) 		PL_get_arg_ex(int n, term_t term, term_t arg);
+COMMON(int) 		PL_get_module_ex(term_t name, Module *m);
 
 
 /* pl-file.c */
@@ -372,6 +373,7 @@ COMMON(term_t) 		PL_new_term_ref__LD(ARG1_LD);
 COMMON(term_t) 		PL_new_term_refs__LD(int n ARG_LD);
 COMMON(int) 		PL_unify__LD(term_t t1, term_t t2 ARG_LD);
 COMMON(int) 		PL_unify_integer__LD(term_t t1, intptr_t i ARG_LD);
+COMMON(int)		PL_unify_int64_ex__LD(term_t t1, int64_t ARG_LD);
 COMMON(int) 		PL_get_atom__LD(term_t t1, atom_t *a ARG_LD);
 COMMON(void) 		PL_put_atom__LD(term_t t1, atom_t a ARG_LD);
 COMMON(void) 		PL_put_integer__LD(term_t t1, long i ARG_LD);
@@ -459,6 +461,7 @@ COMMON(word) 		pl_open_shared_object(term_t file, term_t h, term_t flags);
 COMMON(word) 		pl_close_shared_object(term_t plhandle);
 COMMON(word) 		pl_call_shared_object_function(term_t plhandle, term_t name);
 COMMON(word) 		pl_load_shared_object(term_t file, term_t entry);
+COMMON(void)		cleanupForeign(void);
 
 /* pl-modul.c */
 COMMON(Module) 		lookupModule(atom_t name);
@@ -543,29 +546,16 @@ COMMON(bool) 		can_unify(Word t1, Word t2);
 COMMON(int) 		compareStandard(Word t1, Word t2, int eq ARG_LD);
 COMMON(intptr_t)	skip_list(Word l, Word *tailp ARG_LD);
 COMMON(intptr_t) 	lengthList(term_t list, int errors);
-COMMON(word) 		pl_univ(term_t t, term_t l);
 COMMON(int) 		numberVars(term_t t, nv_options *opts, int n ARG_LD);
 COMMON(word) 		pl_e_free_variables(term_t t, term_t l);
 COMMON(word) 		stringToList(char *s);
-COMMON(word) 		pl_atom_length(term_t w, term_t n);
-COMMON(word) 		pl_name(term_t atom, term_t string);
-COMMON(word) 		pl_atom_chars(term_t atom, term_t string);
-COMMON(word) 		pl_atom_codes(term_t atom, term_t string);
-COMMON(word) 		pl_number_chars(term_t number, term_t string);
-COMMON(word) 		pl_number_codes(term_t number, term_t string);
-COMMON(word) 		pl_char_code(term_t atom, term_t chr);
 COMMON(word) 		pl_atom_concat(term_t a1, term_t a2, term_t a3,
 				       control_t ctx);
-COMMON(word) 		pl_concat_atom(term_t list, term_t atom);
-COMMON(word) 		pl_concat_atom3(term_t list, term_t sep, term_t atom);
 COMMON(word) 		pl_apropos_match(term_t a1, term_t a2);
 COMMON(foreign_t) 	pl_sub_atom(term_t atom,
 				    term_t before, term_t len, term_t after,
 				    term_t sub, control_t h);
-COMMON(word) 		pl_string_length(term_t str, term_t l);
 COMMON(word) 		pl_string_concat(term_t a1, term_t a2, term_t a3, control_t h);
-COMMON(word) 		pl_string_to_atom(term_t str, term_t a);
-COMMON(word) 		pl_string_to_list(term_t str, term_t list);
 COMMON(word) 		pl_sub_string(term_t str,
 			      term_t offset, term_t length, term_t after,
 			      term_t sub, control_t h);
@@ -640,7 +630,8 @@ COMMON(word) 		pl_get_clause_attribute(term_t ref, term_t att, term_t value);
 COMMON(word) 		pl_get_predicate_attribute(term_t pred, term_t k, term_t v);
 COMMON(word) 		pl_set_predicate_attribute(term_t pred, term_t k, term_t v);
 COMMON(void) 		reindexDefinition(Definition def);
-COMMON(void) 		redefineProcedure(Procedure proc, SourceFile sf);
+COMMON(void) 		redefineProcedure(Procedure proc, SourceFile sf,
+					  unsigned int suppress);
 COMMON(void) 		startConsult(SourceFile f);
 COMMON(void) 		indexDefinition(Definition def, long pattern);
 COMMON(word) 		pl_index(term_t pred);
@@ -672,6 +663,7 @@ COMMON(void) 		profRedo(struct call_node *node ARG_LD);
 
 /* pl-read.c */
 COMMON(void) 		resetRead(void);
+COMMON(int)		unicode_separator(pl_wchar_t c);
 COMMON(int) 		str_number(const unsigned char *string,
 				   unsigned char **end,
 				   Number value, bool escape);
@@ -701,6 +693,7 @@ COMMON(word) 		pl_recorded(term_t key, term_t term, term_t ref, control_t h);
 COMMON(word) 		pl_erase(term_t ref);
 COMMON(word) 		pl_term_complexity(term_t t, term_t mx, term_t count);
 COMMON(void) 		undo_while_saving_term(mark *m, Word term);
+COMMON(void)		markAtomsRecord(Record record);
 
 /* pl-rl.c */
 COMMON(void) 		install_rl(void);
@@ -730,10 +723,8 @@ COMMON(void) 		emptyStacks(void);
 COMMON(void) 		freeStacks(ARG1_LD);
 COMMON(void) 		freeLocalData(PL_local_data_t *ld);
 COMMON(word) 		pl_trim_stacks(void);
-COMMON(word) 		pl_limit_stack(term_t s, term_t l);
 COMMON(word) 		pl_stack_parameter(term_t s, term_t k, term_t o, term_t n);
 COMMON(void) 		ensureRoomStack(Stack s, size_t n);
-COMMON(int) 		_PL_get_signum(term_t sig, int *n);
 
 /* pl-sys.c */
 COMMON(word) 		pl_shell(term_t command, term_t status);
@@ -824,7 +815,6 @@ COMMON(word) 		pl_qlf_start_file(term_t name);
 COMMON(word) 		pl_qlf_end_part(void);
 COMMON(word) 		pl_qlf_open(term_t file);
 COMMON(word) 		pl_qlf_close(void);
-COMMON(word) 		pl_qlf_load(term_t file, term_t module);
 COMMON(word) 		pl_qlf_assert_clause(term_t ref, term_t saveclass);
 
 COMMON(void)		wicPutNum(int64_t n, IOSTREAM *fd);
@@ -919,8 +909,6 @@ COMMON(foreign_t) 	pl_open_xterm(term_t title, term_t in, term_t out, term_t err
 
 /* pl-ctype.c */
 
-COMMON(foreign_t) 	pl_downcase_atom(term_t in, term_t out);
-COMMON(foreign_t) 	pl_upcase_atom(term_t in, term_t out);
 COMMON(IOENC)		initEncoding(void);
 COMMON(void) 		initCharTypes(void);
 COMMON(bool) 		systemMode(bool accept);
