@@ -613,7 +613,6 @@ static fid_t
 gvars_to_term_refs(Word **saved_bar_at)
 { GET_LD
   fid_t fid;
-  Word *sb;
 
   *saved_bar_at = NULL;
 
@@ -643,7 +642,10 @@ gvars_to_term_refs(Word **saved_bar_at)
     fid = 0;
 
   if ( LD->frozen_bar )
-  { sb = (Word*)lTop;
+  { Word *sb;
+
+    requireStack(local, sizeof(Word));
+    sb = (Word*)lTop;
     lTop = (LocalFrame)(sb+1);
     *sb = LD->frozen_bar;
     *saved_bar_at = sb;
@@ -695,6 +697,24 @@ term_refs_to_gvars(fid_t fid, Word *saved_bar_at)
 #endif /*O_GVAR*/
 
 
+#ifdef O_CALL_RESIDUE
+static void
+mark_attvars()
+{ GET_LD
+  Word gp;
+
+  for( gp = gBase; gp < gTop; gp += (offset_cell(gp)+1) )
+  { if ( isAttVar(*gp) && !is_marked(gp) )
+    { local_marked--;			/* see mark_variable() */
+      total_marked++;
+      DEBUG(3, Sdprintf("mark_attvars(): marking %p\n", gp));
+      mark_variable(gp PASS_LD);
+    }
+  }
+}
+#endif /*O_CALL_RESIDUE*/
+
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 clearUninitialisedVarsFrame(LocalFrame fr, Code PC);
 
@@ -795,6 +815,14 @@ mark_environments(LocalFrame fr, Code PC)
 		      levelFrame(fr), predicateName(fr->predicate)));
 
     clearUninitialisedVarsFrame(fr, PC);
+#ifdef O_CALL_RESIDUE
+    if ( fr->predicate == PROCEDURE_call_residue_vars2->definition )
+    { if ( !LD->gc.marked_attvars )
+      { mark_attvars();
+	LD->gc.marked_attvars = TRUE;
+      }
+    }
+#endif    
 
     slots  = slotsInFrame(fr, PC);
 #if O_SECURE
@@ -1779,27 +1807,51 @@ collect_phase(LocalFrame fr, Choice ch, Word *saved_bar_at)
 		*********************************/
 
 #if O_DYNAMIC_STACKS
+
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+If s == NULL, consider all stacks
+
+(*) Do not  consider  GC  if  there   are  no  inferences.  This  avoids
+repetetive GC calls while building large   structures  from foreign code
+that calls PL_handle_signals() from time to   time  to enable interrupts
+and call GC.
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
 void
 considerGarbageCollect(Stack s)
 { GET_LD
-  if ( s->gc && trueFeature(GC_FEATURE) && !gc_status.requested )
-  { intptr_t used  = (char *)s->top   - (char *)s->base;
-    intptr_t free  = (char *)s->limit - (char *)s->top;
-    intptr_t limit = (char *)s->limit - (char *)s->base;
 
-    if ( used > s->factor*s->gced_size + s->small )
-    { DEBUG(2, Sdprintf("GC: request on %s, factor=%d, last=%ld, small=%ld\n",
-			s->name, s->factor, s->gced_size, s->small));
-      gc_status.requested = TRUE;
-    } else if ( free < limit/8 && used > s->gced_size + limit/32 ) 
-    { DEBUG(2, Sdprintf("GC: request on low free\n"));
-      gc_status.requested = TRUE;
+  if ( trueFeature(GC_FEATURE) &&
+       !gc_status.requested )
+  { if ( s == NULL )
+    { considerGarbageCollect((Stack)&LD->stacks.global);
+      considerGarbageCollect((Stack)&LD->stacks.trail);
+    } else
+    { if ( s->gc )
+      { intptr_t used  = (char *)s->top   - (char *)s->base;
+	intptr_t free  = (char *)s->limit - (char *)s->top;
+	intptr_t limit = (char *)s->limit - (char *)s->base;
+    
+	if ( LD->gc.inferences == LD->statistics.inferences )
+	{ s->gced_size = used;		/* (*) */
+	  return;
+	}
+    
+	if ( used > s->factor*s->gced_size + s->small )
+	{ DEBUG(2, Sdprintf("GC: request on %s, factor=%d, last=%ld, small=%ld\n",
+			    s->name, s->factor, s->gced_size, s->small));
+	  gc_status.requested = TRUE;
+	} else if ( free < limit/8 && used > s->gced_size + limit/32 ) 
+	{ DEBUG(2, Sdprintf("GC: request on low free\n"));
+	  gc_status.requested = TRUE;
+	}
+    
+	DEBUG(1, if ( gc_status.requested)
+		 { Sdprintf("%s overflow: Posted garbage collect request\n",
+			    s->name);
+		 });
+      }
     }
-
-    DEBUG(1, if ( gc_status.requested)
-	     { Sdprintf("%s overflow: Posted garbage collect request\n",
-			s->name);
-	     });
   }
 }
 #endif /* O_DYNAMIC_STACKS */
@@ -1910,7 +1962,7 @@ check_environments(LocalFrame fr, Code PC, Word key)
     if ( fr->parent )
       fr = fr->parent;
     else
-    { QueryFrame qf = (QueryFrame)addPointer(fr, -offset(queryFrame, frame));
+    { QueryFrame qf = QueryOfTopFrame(fr);
       DEBUG(3, Sdprintf("*** Query %s\n", predicateName(qf->frame.predicate)));
       return qf;
     }
@@ -2152,6 +2204,7 @@ garbageCollect(LocalFrame fr, Choice ch)
   relocation_cells  = 0;
   relocated_cells   = 0;
   local_marked	    = 0;
+  LD->gc.marked_attvars = FALSE;
 
   requireStack(global, sizeof(word));
   requireStack(trail, sizeof(struct trail_entry));
@@ -2211,6 +2264,7 @@ garbageCollect(LocalFrame fr, Choice ch)
 
   unblockGC(PASS_LD1);
   unblockSignals(&mask);
+  LD->gc.inferences = LD->statistics.inferences;
   leaveGC();
 }
 
@@ -2218,7 +2272,6 @@ word
 pl_garbage_collect(term_t d)
 { 
 #if O_DEBUG
-  GET_LD
   int ol = GD->debug_level;
   int nl;
 
