@@ -29,6 +29,10 @@
 #define __SWI_PROLOG__			/* normally defined by the plld compiler driver */
 #endif
 
+#if defined(_MSC_VER) && !defined(__WINDOWS__)
+#define __WINDOWS__ 1
+#endif
+
 #include <stdarg.h>
 #include <stdlib.h>			/* get size_t */
 #include <stddef.h>
@@ -59,7 +63,7 @@ before loading this file.  See end of this file.
 /* PLVERSION: 10000 * <Major> + 100 * <Minor> + <Patch> */
 
 #ifndef PLVERSION
-#define PLVERSION 50647
+#define PLVERSION 50655
 #endif
 
 		 /*******************************
@@ -244,7 +248,7 @@ PL_EXPORT(void *)	PL_foreign_context_address(control_t);
 		*********************************/
 
 typedef struct PL_extension
-{ char 		*predicate_name;	/* Name of the predicate */
+{ const char   *predicate_name;		/* Name of the predicate */
   short		arity;			/* Arity of the predicate */
   pl_function_t	function;		/* Implementing functions */
   short		flags;			/* Or of PL_FA_... */
@@ -255,6 +259,7 @@ typedef struct PL_extension
 #define PL_FA_NONDETERMINISTIC	(0x04)	/* foreign is non-deterministic */
 #define PL_FA_VARARGS		(0x08)	/* call using t0, ac, ctx */
 #define PL_FA_CREF		(0x10)	/* Internal: has clause-reference */
+#define PL_FA_ISO		(0x20)	/* Internal: ISO core predicate */
 
 extern			PL_extension PL_extensions[]; /* not Win32! */
 PL_EXPORT(void)		PL_register_extensions(const PL_extension *e);
@@ -401,6 +406,7 @@ PL_EXPORT(int)		PL_is_functor(term_t t, functor_t f);
 PL_EXPORT(int)		PL_is_list(term_t t);
 PL_EXPORT(int)		PL_is_atomic(term_t t);
 PL_EXPORT(int)		PL_is_number(term_t t);
+PL_EXPORT(int)		PL_is_acyclic(term_t t);
 
 			/* Assign to term-references */
 PL_EXPORT(void)		PL_put_variable(term_t t);
@@ -668,6 +674,9 @@ UNICODE file functions.
 #define REP_FN		REP_MB
 #endif
 
+#define PL_DIFF_LIST	0x20000		/* PL_unify_chars() */
+
+
 #ifdef SIO_MAGIC			/* defined from <SWI-Stream.h> */
 		 /*******************************
 		 *	  STREAM SUPPORT	*
@@ -738,6 +747,7 @@ install_t		PL_install_readline(void);
 #endif
 PL_EXPORT(int)		PL_toplevel(void);
 PL_EXPORT(int)		PL_cleanup(int status);
+PL_EXPORT(void)		PL_cleanup_fork();
 PL_EXPORT(void)		PL_halt(int status) NORETURN;
 
 		 /*******************************
@@ -808,6 +818,8 @@ PL_EXPORT(void) (*PL_signal(int sig, void (*func)(int)))(int);
 PL_EXPORT(void) PL_interrupt(int sig);
 PL_EXPORT(int)  PL_raise(int sig);
 PL_EXPORT(int)  PL_handle_signals(void);
+PL_EXPORT(int) 	PL_get_signum_ex(term_t sig, int *n);
+
 
 		/********************************
 		*      PROLOG ACTION/QUERY      *
