@@ -280,6 +280,13 @@ signal_name(int sig)
 static int
 signal_index(const char *name)
 { struct signame *sn = signames;
+  char tmp[12];
+
+  if ( strncmp(name, "SIG", 3) == 0 && strlen(name) < 12 )
+  { strcpy(tmp, name+3);
+    strlwr(tmp);
+    name = tmp;
+  }
 
   for( ; sn->name; sn++ )
   { if ( streq(sn->name, name) )
@@ -291,20 +298,24 @@ signal_index(const char *name)
 
 
 int
-_PL_get_signum(term_t sig, int *n)
+PL_get_signum_ex(term_t sig, int *n)
 { char *s;
   int i = -1;
 
-  if ( !PL_get_integer(sig, &i) )
-  { if ( PL_get_atom_chars(sig, &s) )
-      i = signal_index(s);
+  if ( PL_get_integer(sig, &i) )
+  { 
+  } else if ( PL_get_chars(sig, &s, CVT_ATOM) )
+  { i = signal_index(s);
+  } else
+  { return PL_error(NULL, 0, NULL, ERR_TYPE, ATOM_signal, sig);
   }
+
   if ( i > 0 && i < 32 )		/* where to get these? */
   { *n = i;
     return TRUE;
   }
 
-  return FALSE;
+  return PL_error(NULL, 0, NULL, ERR_DOMAIN, ATOM_signal, sig);
 }
 
 
@@ -347,6 +358,7 @@ dispatch_signal(int sig, int sync)
   fid_t fid;
   uintptr_t lTopSave;
   int saved_current_signal;
+  int saved_sync;
 
 #ifdef O_PLMT
   if ( !LD )
@@ -367,6 +379,7 @@ dispatch_signal(int sig, int sync)
 
   lTopSave = (char*)lTop - (char*)lBase;
   saved_current_signal = LD->current_signal;
+  saved_sync = LD->sync_signal;
 
   switch(sig)
   { case SIGFPE:
@@ -390,6 +403,7 @@ dispatch_signal(int sig, int sync)
 
   blockGC(PASS_LD1);
   LD->current_signal = sig;
+  LD->sync_signal = sync;
   fid = PL_open_signal_foreign_frame();
 
   DEBUG(1, Sdprintf("Handling signal %d, pred = %p, handler = %p\n",
@@ -440,6 +454,7 @@ dispatch_signal(int sig, int sync)
   }
 
   LD->current_signal = saved_current_signal;
+  LD->sync_signal = saved_sync;
   PL_discard_foreign_frame(fid);
   lTop = addPointer(lBase, lTopSave);
 
@@ -782,6 +797,12 @@ PL_handle_signals()
 
   if ( exception_term )
     return -1;
+
+  if ( gc_status.requested )
+  { DEBUG(1, Sprintf("GC from PL_handle_signals()\n"));
+    garbageCollect(NULL, NULL);
+  }
+
   return done;
 }
 
@@ -911,6 +932,8 @@ pl-thread.c to create stacks for Prolog threads.
 
 allocStacks() does the  real  work   and  has  several  implementations,
 depending on the OS features.
+
+Requested stack sizes are in bytes.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 int
@@ -1186,7 +1209,9 @@ unmap(Stack s)
   { intptr_t len = (char *)s->max - (char *)addr;
 
 #ifdef MAP_FIXED
+#ifndef MMAP_OVERRIDES
     munmap(addr, len);
+#endif
     if ( mmap(addr, len, MAPPROTECT, MAP_FIXED|MAP_FLAGS, mapfd, 0L) != addr )
       fatalError("Failed to remap 0x%x bytes at %p: %s",
 		 len, addr, OsError());
@@ -1737,9 +1762,6 @@ trimStacks(ARG1_LD)
 #endif /*O_DYNAMIC_STACKS*/
 #endif /*O_SHIFT_STACKS*/
 
-  LD->stacks.global.gced_size = usedStack(global);
-  LD->stacks.trail.gced_size  = usedStack(trail);
-
   for(te = tTop; --te >= tBase; )
   { Word p = te->address;
     
