@@ -516,10 +516,6 @@ update_linepos(IOSTREAM *s, int c)
       p->lineno++;
       p->linepos = 0;
       s->flags &= ~SIO_NOLINEPOS;
-#ifdef __WINDOWS__
-      if ( s->flags & O_TEXT )
-	p->charno++;			/* writes one extra! */
-#endif
       break;
     case '\r':
       p->linepos = 0;
@@ -964,8 +960,7 @@ retry:
   }
 
 out:
-  if ( c == '\r' && (s->flags&SIO_TEXT) &&
-       !(s->flags&SIO_ISATTY) )
+  if ( c == '\r' && (s->flags&SIO_TEXT) )
   { switch(s->newline)
     { case SIO_NL_DETECT:
 	s->newline = SIO_NL_DOS;
@@ -3223,13 +3218,15 @@ SinitStreams()
       S__iob[i].mutex = malloc(sizeof(recursiveMutex));
       recursiveMutexInit(S__iob[i].mutex);
 #endif
-#ifdef __WINDOWS__
-      pt_init();
-#endif
 #if CRLF_MAPPING
+      _setmode(i, O_BINARY);
       S__iob[i].newline = SIO_NL_DOS;
 #endif
     }
+
+#ifdef __WINDOWS__
+    pt_init();				/* init popen() issues */
+#endif
   }
 }
 
@@ -3287,11 +3284,17 @@ Sclosehook(void (*hook)(IOSTREAM *s))
 
 void
 Sreset(void)
-{ IOSTREAM *s = Sinput;
+{ IOSTREAM *s;
 
-  if ( s && s->magic == SIO_MAGIC )
+  if ( (s=Sinput) && s->magic == SIO_MAGIC )
   { s->bufp = s->limitp = s->buffer;
   }
+  if ( (s=Soutput) && s->magic == SIO_MAGIC )
+  { s->bufp = s->buffer;
+  }
+  if ( (s=Serror) && s->magic == SIO_MAGIC )
+  { s->bufp = s->buffer;
+  }
 } 
 
 
