@@ -139,11 +139,12 @@ extern counting_mutex _PL_mutexes[];	/* Prolog mutexes */
 #define L_INIT	       14
 #define L_TERM	       15
 #define L_GC	       16
-#define L_FOREIGN      17
-#define L_OS	       18
+#define L_AGC	       17
+#define L_FOREIGN      18
+#define L_OS	       19
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-The IFMT(id, g) macro  is  used  to   bypass  mutexes  if  threading  is
+The IF_MT(id, g) macro  is  used  to   bypass  mutexes  if  threading  is
 disabled. We cannot do this for the L_THREAD mutex however as we need to
 control when threads can be created.
 
@@ -151,7 +152,7 @@ We  assume  id  ==  L_THREAD  is  optimized  away  if  id  is  known  at
 compile-time
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-#define IFMT(id, g) if ( id == L_THREAD || GD->thread.enabled ) g
+#define IF_MT(id, g) if ( id == L_THREAD || GD->thread.enabled ) g
 
 #ifdef O_CONTENTION_STATISTICS
 #define countingMutexLock(cm) \
@@ -182,17 +183,17 @@ compile-time
 	do { Sdprintf("[%s] %s:%d: LOCK(%s)\n", \
 		      threadName(0), \
 		      __BASE_FILE__, __LINE__, #id); \
-             IFMT(id, countingMutexLock(&_PL_mutexes[id])); \
+             IF_MT(id, countingMutexLock(&_PL_mutexes[id])); \
 	   } while(0)
 #define PL_UNLOCK(id) \
 	do { Sdprintf("[%s] %s:%d: UNLOCK(%s)\n", \
 		      threadName(0), \
 		      __BASE_FILE__, __LINE__, #id); \
-	     IFMT(id, countingMutexUnlock(&_PL_mutexes[id])); \
+	     IF_MT(id, countingMutexUnlock(&_PL_mutexes[id])); \
 	   } while(0)
 #else
-#define PL_LOCK(id)   IFMT(id, countingMutexLock(&_PL_mutexes[id]))
-#define PL_UNLOCK(id) IFMT(id, countingMutexUnlock(&_PL_mutexes[id]))
+#define PL_LOCK(id)   IF_MT(id, countingMutexLock(&_PL_mutexes[id]))
+#define PL_UNLOCK(id) IF_MT(id, countingMutexUnlock(&_PL_mutexes[id]))
 #endif
 
 #define LOCKDEF(def) \
@@ -306,10 +307,6 @@ extern word		pl_thread_create(term_t goal, term_t id,
 					 term_t options);
 extern word		pl_thread_join(term_t thread, term_t retcode);
 extern word		pl_thread_exit(term_t retcode);
-extern word		pl_thread_kill(term_t thread, term_t sig);
-extern word		pl_thread_send_message(term_t thread, term_t msg);
-extern word		pl_thread_get_message(term_t msg);
-extern word		pl_thread_peek_message(term_t msg);
 extern foreign_t	pl_thread_signal(term_t thread, term_t goal);
 
 extern foreign_t	pl_thread_at_exit(term_t goal);
@@ -340,6 +337,7 @@ intptr_t			system_thread_id(PL_thread_info_t *info);
 void		forThreadLocalData(void (*func)(struct PL_local_data *),
 				   unsigned flags);
 void		resumeThreads(void);
+void		markAtomsThreads(void);
 
 #define PL_THREAD_SUSPEND_AFTER_WORK	0x1 /* forThreadLocalData() */
 
@@ -438,5 +436,7 @@ extern void		initPrologThreads(void);
 #define PL_is_atom(t)		PL_is_atom__LD(t PASS_LD)
 #define PL_unify_list(l, h, t)	PL_unify_list__LD(l, h, t PASS_LD)
 #define PL_cons_list(l, h, t)	PL_cons_list__LD(l, h, t PASS_LD)
+#define PL_get_text(l, t, f)	PL_get_text__LD(l, t, f PASS_LD)
+#define PL_unify_int64_ex(t, i)	PL_unify_int64_ex__LD(t, i PASS_LD)
 
 #endif /*PL_THREAD_H_DEFINED*/
