@@ -1097,7 +1097,7 @@ findQuery(LocalFrame fr)
     fr = fr->parent;
 
   if ( fr )
-    return (QueryFrame)addPointer(fr, -offset(queryFrame, frame));
+    return QueryOfTopFrame(fr);
   return NULL;
 }
 
@@ -1237,13 +1237,13 @@ helpInterrupt(void)
 static void
 interruptHandler(int sig)
 { int c; 
+  int safe;
 
   if ( !GD->initialised )
   { Sfprintf(Serror, "Interrupt during startup. Cannot continue\n");
     PL_halt(1);
   }  
 
-again:
 #ifdef O_PLMT
   if ( !LD )				/* we can't handle this; main thread */
   { PL_thread_raise(1, sig);		/* should try to do this */
@@ -1258,8 +1258,31 @@ again:
   }
 #endif
 
-  Sfputs("\nAction (h for help) ? ", Sdout);
-  Sflush(Sdout);
+#if __unix__				/* actually, asynchronous signal handling */
+  if ( !LD->sync_signal )
+  { long mask = (1L<<(sig-1));
+
+    if ( (LD->pending_signals & mask) )
+    { LD->pending_signals &= ~mask;
+      safe = FALSE;
+    } else
+    { DEBUG(1, Sdprintf("Reposting as synchronous\n"));
+      PL_raise(sig);
+      return;
+    }
+  } else
+  { safe = TRUE;
+  }
+#endif
+
+  Sreset();
+again:
+  if ( safe )
+  { printMessage(ATOM_debug, PL_FUNCTOR, FUNCTOR_interrupt1, PL_ATOM, ATOM_begin);
+  } else
+  { Sfprintf(Sdout, "\n%sAction (h for help) ? ", safe ? "" : "[forced] ");
+    Sflush(Sdout);
+  }
   ResetTty();                           /* clear pending input -- atoenne -- */
   c = getSingleChar(Sdin, FALSE);
 
@@ -1272,7 +1295,11 @@ again:
 		unblockSignal(sig);	/* into pl_break() itself */
 		pl_break();
 		goto again;		
-    case 'c':	Sfputs("continue\n", Sdout);
+    case 'c':	if ( safe )
+		{ printMessage(ATOM_debug, PL_FUNCTOR, FUNCTOR_interrupt1, PL_ATOM, ATOM_end);
+		} else
+		{ Sfputs("continue\n", Sdout);
+		}
 		break;
     case 04:
     case EOF:	Sfputs("EOF: ", Sdout);
@@ -1289,6 +1316,8 @@ again:
 		goto again;
 #ifdef O_DEBUGGER
     case 't':	Sfputs("trace\n", Sdout);
+		if ( safe )
+		  printMessage(ATOM_debug, PL_FUNCTOR, FUNCTOR_interrupt1, PL_ATOM, ATOM_trace);
 		pl_trace();
 		break;
 #endif /*O_DEBUGGER*/
@@ -1375,8 +1404,7 @@ debugmode(debug_type doit, debug_type *old)
 	{ if ( fr->parent )
 	    fr = fr->parent;
 	  else
-	  { QueryFrame qf = (QueryFrame)addPointer(fr, -offset(queryFrame,
-							       frame));
+	  { QueryFrame qf = QueryOfTopFrame(fr);
 	    assert(qf->magic == QID_MAGIC);
 	    qf->debugSave = DBG_ON;
 	    fr = qf->saved_environment;
