@@ -958,7 +958,7 @@ loadPredicate(IOSTREAM *fd, int skip ARG_LD)
   def = proc->definition;
   if ( !skip && currentSource )
   { if ( def->definition.clauses )
-      redefineProcedure(proc, currentSource);
+      redefineProcedure(proc, currentSource, DISCONTIGUOUS_STYLE);
     addProcedureSourceFile(currentSource, proc);
   }
   if ( def->references == 0 && !def->hash_info )
@@ -1337,8 +1337,9 @@ Note that we keep track of the `current procedure' to keep  all  clauses
 of a predicate together.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-static Table savedXRTable;		/* saved XR entries */
-static intptr_t  savedXRTableId;		/* next id */
+static Table	savedXRTable;		/* saved XR entries */
+static intptr_t	savedXRTableId;		/* next id */
+static intptr_t registered_atoms;	/* safety check */
 
 #define STR_NOLEN ((size_t)-1)
 
@@ -1473,6 +1474,40 @@ putLong(long v, IOSTREAM *fd)		/* always 4 bytes */
 }
 
 
+static void
+freeXRSymbol(Symbol s)
+{ word w = (word)s->name;
+
+  if ( w&0x1 )
+  { w &= ~0x1;
+    if ( isAtom(w) )
+    { registered_atoms--;
+      PL_unregister_atom(w);
+      DEBUG(5, Sdprintf("UNREG: %s\n", stringAtom(w)));
+    }
+  }
+}
+
+
+void
+initXR()
+{ currentProc		    = NULL;
+  currentSource		    = NULL;
+  savedXRTable		    = newHTable(256);
+  savedXRTable->free_symbol = freeXRSymbol;
+  savedXRTableId	    = 0;
+  registered_atoms	    = 0;
+}
+
+
+void
+destroyXR()
+{ destroyHTable(savedXRTable);
+  savedXRTable = NULL;
+  assert(registered_atoms==0);
+}
+
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 XR (External Reference)  table  handling.   The  table  contains  atoms,
 functors  and  various  types  of    pointers   (Module,  Procedure  and
@@ -1481,7 +1516,8 @@ conflict with pointers. We assume -as in  many other places in the code-
 that pointers are 4-byte aligned.
 
 savedXRConstant()  must  be  used  for    atom_t  and  functor_t,  while
-savedXRPointer must be used for the pointers.
+savedXRPointer  must  be  used  for   the    pointers.   The  value  for
+savedXRConstant() is or-ed with 0x1 to avoid conflict with pointers.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 static int
@@ -1506,10 +1542,17 @@ savedXR(void *xr, IOSTREAM *fd)
 
 static inline int
 savedXRConstant(word w, IOSTREAM *fd)
-{ assert(tag(w) == TAG_ATOM);		/* Only functor_t and atom_t */
-  w |= 0x1;				/* Avoid conflict with pointer */
-  
-  return savedXR((void *)w, fd);
+{ int rc;
+
+  assert(tag(w) == TAG_ATOM);		/* Only functor_t and atom_t */
+
+  if ( !(rc=savedXR((void *)(w|0x1), fd)) && isAtom(w) )
+  { registered_atoms++;
+    DEBUG(5, Sdprintf("REG: %s\n", stringAtom(w)));
+    PL_register_atom(w);
+  }
+
+  return rc;
 }
 
 
@@ -1889,10 +1932,7 @@ writeWicHeader(IOSTREAM *fd)
   else
     putString("<no home>",  STR_NOLEN, fd);
 
-  currentProc    = (Procedure) NULL;
-  currentSource  = (SourceFile) NULL;
-  savedXRTable   = newHTable(256);
-  savedXRTableId = 0;
+  initXR();
 
   DEBUG(2, Sdprintf("Header complete ...\n"));
   succeed;
@@ -1906,8 +1946,7 @@ writeWicTrailer(IOSTREAM *fd)
 
   closeProcedureWic(fd);
   Sputc('X', fd);
-  destroyHTable(savedXRTable);
-  savedXRTable = NULL;
+  destroyXR();
   Sputc('T', fd);
 
   wicFd = NULL;
@@ -2140,11 +2179,7 @@ qlfOpen(atom_t name)
   putNum(sizeof(word)*8, wicFd);
 
   putString(absname, STR_NOLEN, wicFd);
-
-  currentProc    = (Procedure) NULL;
-  currentSource  = (SourceFile) NULL;
-  savedXRTable   = newHTable(256);
-  savedXRTableId = 0;
+  initXR();
   initSourceMarks();
 
   succeed;
@@ -2160,9 +2195,7 @@ qlfClose(ARG1_LD)
   Sclose(fd);
   wicFd = NULL;
   mkWicFile = NULL;
-
-  destroyHTable(savedXRTable);
-  savedXRTable = NULL;
+  destroyXR();
   
   succeed;
 }
@@ -2245,28 +2278,33 @@ popPathTranslation()
 }
 
 static bool
-qlfLoad(char *file, Module *module ARG_LD)
-{ IOSTREAM *fd;
-  bool rval;
+qlfLoad(IOSTREAM *fd, Module *module ARG_LD)
+{ bool rval;
   int lversion;
-  char *absloadname;
+  const char *absloadname;
   char tmp[MAXPATHLEN];
   int saved_wsize;
+  atom_t file;
 
-  wicFile = file;
-  if ( !(absloadname = AbsoluteFile(wicFile, tmp)) )
-    fail;
-  
-  if ( !(fd = Sopen_file(file, "rbr")) )
-  { term_t f = PL_new_term_ref();
+  if ( (file = fileNameStream(fd)) )
+  { PL_chars_t text;
 
-    PL_put_atom_chars(f, file);
-    return PL_error(NULL, 0, OsError(), ERR_FILE_OPERATION,
-		    ATOM_open, ATOM_source_sink, f);
+    if ( !get_atom_text(file, &text) )
+      fail;
+    if ( !PL_mb_text(&text, REP_FN) )
+    { PL_free_text(&text);
+      fail;
+    }
+    wicFile = text.text.t;
+    if ( !(absloadname = AbsoluteFile(wicFile, tmp)) )
+      fail;
+    PL_free_text(&text);
+  } else
+  { absloadname = NULL;
   }
+  
   if ( !(lversion = qlfVersion(fd)) || lversion < LOADVERSION )
-  { Sclose(fd);
-    if ( lversion )
+  { if ( lversion )
       warning("$qlf_load/1: %s bad version (file version = %d, prolog = %d)",
 	      wicFile, lversion, VERSION);
     fail;
@@ -2288,8 +2326,6 @@ qlfLoad(char *file, Module *module ARG_LD)
   popXrIdTable(PASS_LD1);
   popPathTranslation();
 
-  Sclose(fd);
-
   return rval;
 }
 
@@ -2373,8 +2409,8 @@ pl_qlf_start_module(term_t name)
   { GET_LD
     Module m;
 
-    if ( !PL_get_module(name, &m) )
-      return warning("qlf_start_module/1: argument must be an atom");
+    if ( !PL_get_module_ex(name, &m) )
+      fail;
   
     return qlfStartModule(m, wicFd PASS_LD);
   }
@@ -2389,8 +2425,8 @@ pl_qlf_start_sub_module(term_t name)
   { GET_LD
     Module m;
 
-    if ( !PL_get_module(name, &m) )
-      return warning("qlf_start_sub_module/1: argument must be an atom");
+    if ( !PL_get_module_ex(name, &m) )
+      fail;
   
     return qlfStartSubModule(m, wicFd PASS_LD);
   }
@@ -2443,31 +2479,43 @@ pl_qlf_close()
 }
 
 
-word
-pl_qlf_load(term_t file, term_t module)
+/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+$qlf_load(:Stream, -ModuleOut)
+- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
+
+static
+PRED_IMPL("$qlf_load", 2, qlf_load, PL_FA_TRANSPARENT)
 { GET_LD
+  term_t qstream = A1;
+  term_t module = A2;
   Module m, oldsrc = LD->modules.source;
   char *fn;
   bool rval;
-  term_t name = PL_new_term_ref();
+  term_t stream = PL_new_term_ref();
+  IOSTREAM *fd;
+  IOENC saved_enc;
 
   m = oldsrc;
-  if ( !PL_strip_module(file, &m, name) )
+  if ( !PL_strip_module(qstream, &m, stream) )
     fail;
-  if ( !PL_get_file_name(name, &fn, 0) )
+  if ( !PL_get_stream_handle(stream, &fd) )
     fail;
 
+  saved_enc = fd->encoding;
+  fd->encoding = ENC_OCTET;
   LD->modules.source = m;
-  rval = qlfLoad(fn, &m PASS_LD);
+  rval = qlfLoad(fd, &m PASS_LD);
   LD->modules.source = oldsrc;
+  fd->encoding = saved_enc;
 
-  if ( !rval )
-    fail;
+  if ( rval )
+  { if ( m )
+      return PL_unify_atom(module, m->name);
 
-  if ( m )
-    return PL_unify_atom(module, m->name);
-  else
     return PL_unify_integer(module, 0);
+  }
+
+  fail;
 }
 
 
@@ -2520,17 +2568,17 @@ pl_add_directive_wic(term_t term)
 
 
 word
-pl_import_wic(term_t module, term_t head)
+pl_import_wic(term_t module, term_t pi)
 { if ( wicFd )
   { GET_LD
-    Module m;
-    functor_t f;
+    Module m = NULL;
+    functor_t fd;
 
     if ( !PL_get_module(module, &m) ||
-	 !PL_get_functor(head, &f) )
-      return warning("$import_wic/3: instantiation fault");
+	 !get_functor(pi, &fd, &m, 0, GF_PROCEDURE) )
+      fail;
 
-    return importWic(lookupProcedure(f, m), wicFd PASS_LD);
+    return importWic(lookupProcedure(fd, m), wicFd PASS_LD);
   }
 
   succeed;
@@ -2720,7 +2768,7 @@ qlfCleanup()
   if ( getstr_buffer )
   { free(getstr_buffer);
     getstr_buffer = NULL;
-    getstr_buffer_size = 512;
+    getstr_buffer_size = 0;
   }
 }
 
@@ -2752,4 +2800,5 @@ wicPutStringW(const pl_wchar_t *w, size_t len, IOSTREAM *fd)
 
 BeginPredDefs(wic)
   PRED_DEF("$qlf_info", 5, qlf_info, 0)
+  PRED_DEF("$qlf_load", 2, qlf_load, PL_FA_TRANSPARENT)
 EndPredDefs
